################################################################################
#
#       This file is part of Gato (Graph Animation Toolbox) 
#       You can find more information at 
#       http://gato.sf.net
#
#	file:   Embedder.py
#	author: Ramazan Buzdemir
#
#       Copyright (C) 1998-2015, Alexander Schliep, Winfried Hochstaettler and 
#       Copyright 1998-2001 ZAIK/ZPR, Universitaet zu Koeln
#                                   
#       Contact: alexander@schliep.org, winfried.hochstaettler@fernuni-hagen.de             
#
#       Information: http://gato.sf.net
#
#       This library is free software; you can redistribute it and/or
#       modify it under the terms of the GNU Library General Public
#       License as published by the Free Software Foundation; either
#       version 2 of the License, or (at your option) any later version.
#
#       This library is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#       Library General Public License for more details.
#
#       You should have received a copy of the GNU Library General Public
#       License along with this library; if not, write to the Free
#       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#
#       This file is version $Revision: 670 $ 
#                       from $Date: 2015-01-13 16:04:11 -0500 (Tue, 13 Jan 2015) $
#             last change by $Author: schliep $.
#
################################################################################

from GatoGlobals import gInfinity

class Rect:
    def __init__(self):
        self.x1 = 30
        self.y1 = 30
        self.x2 = 600
        self.y2 = 400

    def midpoint(self):
        return ((self.x2 - self.x1)/2.0, (self.y2 - self.y1)/2.0)  

    def height(self):
        return self.x2 - self.x1

    def width(self):
        return self.y2 - self.y1
            

class Embedder:
    """ This class provides an abstract Embedder as
        a base for actual Embedder implementations """
    
    def Name(self):
        """ Return a short descriptive name for the embedder e.g. usable as 
            a menu item """
        return "none"
        
    def Embed(self, theGraphEditor):
        """ Compute the Embedding. Changed display through theGraphEditor.
            Return value != none designates error/warning message """
        return none
        
def RedrawGraph(theGraphEditor):
    theGraphEditor.SetGraphMenuGrid(0)
    for v in theGraphEditor.G.vertices:
        theGraphEditor.MoveVertex(v, theGraphEditor.G.xCoord[v],
                                  theGraphEditor.G.yCoord[v], 1)
        
#----------------------------------------------------------------------
import random

def RandomCoords(G,rect=None):
    if rect is None:
        rect = Rect()
    G.xCoord={}
    G.yCoord={}
    for v in G.vertices:
        G.xCoord[v]=random.randint(rect.x1,rect.x2)
        G.yCoord[v]=random.randint(rect.y1,rect.y2)
    return 1

    
class RandomEmbedder(Embedder):

    def Name(self):
        return "Randomize Layout"
        
    def Embed(self, theGraphEditor,rect=None):
        if theGraphEditor.G.Order()==0:
            return
            
        theGraphEditor.config(cursor="watch")
        theGraphEditor.update()

        if rect is None:
            rect = Rect()
        if RandomCoords(theGraphEditor.G,rect):
            RedrawGraph(theGraphEditor)
            theGraphEditor.dirty = 1

        theGraphEditor.config(cursor="")
        
#----------------------------------------------------------------------
from math import pi, sin, cos

def CircularCoords(G,rect=None):
    if rect is None:
        rect = Rect()    
    G.xCoord={}
    G.yCoord={}
    distance = 2*pi/G.Order()
    degree = 0
    xMiddle, yMiddle = rect.midpoint()
    radius = min(rect.height(), rect.width()) / 2.0 - 50
    for v in G.vertices:
        G.xCoord[v]=radius*cos(degree)+xMiddle
        G.yCoord[v]=radius*sin(degree)+yMiddle
        degree=degree+distance
    return 1
    
class CircularEmbedder(Embedder):

    def Name(self):
        return "Circular Layout"
        
    def Embed(self, theGraphEditor,rect=None):
        if theGraphEditor.G.Order()==0:
            return
            
        theGraphEditor.config(cursor="watch")
        theGraphEditor.update()

        if CircularCoords(theGraphEditor.G,rect):
            RedrawGraph(theGraphEditor)
            theGraphEditor.dirty = 1

        theGraphEditor.config(cursor="")
        
        #----------------------------------------------------------------------
from PlanarEmbedding import *

class FPP_PlanarEmbedder(Embedder):

    def Name(self):
        return "Planar Layout (FPP)"
        
    def Embed(self, theGraphEditor):
    
        theGraphEditor.config(cursor="watch")
        theGraphEditor.update()
        
        if theGraphEditor.G.Order()==0:
            return
        if FPP_PlanarCoords(theGraphEditor.G):
            RedrawGraph(theGraphEditor)
            theGraphEditor.dirty = 1

        theGraphEditor.config(cursor="")
        
class Schnyder_PlanarEmbedder(Embedder):

    def Name(self):
        return "Planar Layout (Schnyder)"
        
    def Embed(self, theGraphEditor):
        if theGraphEditor.G.Order()==0:
            return
            
        theGraphEditor.config(cursor="watch")
        theGraphEditor.update()
        
        if Schnyder_PlanarCoords(theGraphEditor.G):
            RedrawGraph(theGraphEditor)
            theGraphEditor.dirty = 1

        theGraphEditor.config(cursor="")
        
        #----------------------------------------------------------------------
from Tkinter import *
import tkSimpleDialog 
import string
from tkMessageBox import showwarning

from DataStructures import Stack

"""
def center(G):

    # Floyd-Algorithm
    INFTY=9999999
    dist={}
    for v in G.vertices:
        for w in G.vertices:
            if w in G.InOutNeighbors(v):
                dist[v,w]=1
            elif v==w: 
                dist[v,w]=0
            else:
                dist[v,w]=INFTY

    for u in G.vertices:
        for v in G.vertices:
            for w in G.vertices:		
                if dist[v,u]+dist[u,w]<dist[v,w]:
                    dist[v,w]=dist[v,u]+dist[u,w]

    max1=INFTY
    center=G.vertices[0]
    for u in G.vertices:
        max2=0
        for v in G.vertices:
            if dist[u,v]>max2:
                max2=dist[u,v]
        if max2<max1: 
            center=u
            max1=max2

    return center
"""

class TreeLayoutDialog(tkSimpleDialog.Dialog):

    def __init__(self, master):
        self.G = master.G
        tkSimpleDialog.Dialog.__init__(self, master, "Tree Layout")
        
        
    def body(self, master):
        self.resizable(0,0)
        
        self.root=StringVar()
        self.root.set(self.G.vertices[0])
        #self.root.set(center(self.G))
        label = Label(master, text="root :", anchor=W)
        label.grid(row=0, column=0, padx=0, pady=2, sticky="w")
        entry=Entry(master, width=6, exportselection=FALSE,textvariable=self.root)
        entry.selection_range(0,"end")
        entry.focus_set()
        entry.grid(row=0,column=1, padx=2, pady=2, sticky="w")
        
        self.orientation=StringVar()
        self.orientation.set("vertical")
        
        radio=Radiobutton(master, text="vertical", variable=self.orientation, 
                          value="vertical")
        radio.grid(row=0, column=2, padx=2, pady=2, sticky="w") 
        radio=Radiobutton(master, text="horizontal", variable=self.orientation,
                          value="horizontal")
        radio.grid(row=1, column=2, padx=2, pady=2, sticky="w") 
        
    def validate(self):
        try: 
            if (string.atoi(self.root.get())<0 or 
                string.atoi(self.root.get()) not in self.G.vertices):
                raise rootError
            self.result=[]
            self.result.append(string.atoi(self.root.get()))
            self.result.append(self.orientation.get())
            return self.result
        except:
            showwarning("Warning", 
                        "Invalid root !!!\n"
                        "Please try again !")
            return 0
            
            
def TreeCoords(G, root, orientation):
    S = Stack()
    visited = {}
    d = {}
    leaves = []
    number_of_leaves = 0
    height = 0
    nodes = {}
    children = {}
    father = {}
    
    for v in G.vertices:
        visited[v] = 0	
    visited[root] = 1
    S.Push(root)
    d[root] = 0
    nodes[0] = []
    children[root] = []
    father[root] = None 
    
    while S.IsNotEmpty():
        v = S.Pop()
        if orientation=="vertical":
            nodes[d[v]].insert(0,v)
            if v!=root: children[father[v]].insert(0,v)
        else:
            nodes[d[v]].append(v)
            if v!=root: children[father[v]].append(v)
        isleaf = 1
        for w in G.InOutNeighbors(v):
            if visited[w] == 0:
                isleaf = 0
                visited[w] = 1
                d[w] = d[v] + 1
                children[w] = []
                father[w] = v
                if d[w]>height:
                    height = d[w]
                    nodes[height] = []
                S.Push(w)
        if isleaf:
            number_of_leaves = number_of_leaves + 1
            if orientation=="vertical":
                leaves.insert(0,v)
            else:
                leaves.append(v)
                
                # Test whether the graph is connected and
                # acyclic.(=test whether the graph is a tree)
    for v in G.vertices:
    
        if visited[v]==0:
            showwarning("Warning", 
                        "Graph is not a tree,\n"
                        "not connected !!!")
            return 0
            
        ch_len = len(children[v])
        if v!=root: ch_len = ch_len + 1
        if ch_len<len(G.InOutNeighbors(v)):
            showwarning("Warning", 
                        "Graph is not a tree,\n"
                        "contains cycles !!!")                
            return 0
            
            
    if number_of_leaves<=19:
        dist1 = 50
    else:
        dist1 = 900 / (number_of_leaves-1)
        
    if height+1<=19:
        dist2 = 50
    else:
        dist2 = 900 / height
        
    if dist1<25 or dist2<30: 
        showwarning("Warning", 
                    "Tree-Layout not possible,\n"
                    "the tree is too large !!!")
        return 0
        
        
    Coord1 = {}
    Coord2 = {}
    i = 0
    for v in leaves:
        Coord1[v] = 50 + i * dist1
        Coord2[v] = 50 + d[v] * dist2
        i = i + 1
        
    i = height - 1
    while i>=0:
        for v in nodes[i]:
            if children[v]!=[]:
                Coord2[v] = 50 + d[v] * dist2
                if len(children[v])==1:
                    Coord1[v] = Coord1[children[v][0]]
                else:
                    Coord1[v] = ( Coord1[children[v][0]] +
                                  (Coord1[children[v][-1]] - 
                                   Coord1[children[v][0]]) / 2)  
        i=i-1
        
    if orientation=="vertical":
        G.xCoord=Coord1
        G.yCoord=Coord2
    else:
        G.xCoord=Coord2
        G.yCoord=Coord1
        
    return 1
    
    
class TreeEmbedder(Embedder):

    def Name(self):
        return "Tree Layout"
        
    def Embed(self, theGraphEditor):
        if theGraphEditor.G.Order()==0:
            return
            
        theGraphEditor.config(cursor="watch")
        
        dial = TreeLayoutDialog(theGraphEditor)
        if dial.result is None:
            theGraphEditor.config(cursor="")
            return	
            
        if TreeCoords(theGraphEditor.G, dial.result[0], dial.result[1]):
            RedrawGraph(theGraphEditor)
            theGraphEditor.dirty = 1

        theGraphEditor.config(cursor="")
        
        #----------------------------------------------------------------------
from GraphUtil import BFS

class BFSLayoutDialog(tkSimpleDialog.Dialog):

    def __init__(self, master):
        self.G = master.G
        tkSimpleDialog.Dialog.__init__(self, master, "BFS Layout")
        
        
    def body(self, master):
        self.resizable(0,0)
        
        self.root=StringVar()
        self.root.set(self.G.vertices[0])
        label = Label(master, text="root :" , anchor=W)
        label.grid(row=0, column=0, padx=0, pady=2, sticky="w")
        entry=Entry(master, width=6, exportselection=FALSE,textvariable=self.root)
        entry.selection_range(0,"end")
        entry.focus_set()
        entry.grid(row=0,column=1, padx=2, pady=2, sticky="w")
        
        self.direction=StringVar()
        self.direction.set("forward")
        if self.G.QDirected():
            radio=Radiobutton(master, text="forward", variable=self.direction, 
                               value="forward")
            radio.grid(row=0, column=2, padx=2, pady=2, sticky="w") 
            radio=Radiobutton(master, text="backward", variable=self.direction,
                               value="backward")
            radio.grid(row=1, column=2, padx=2, pady=2, sticky="w") 
            
            
    def validate(self):
        try: 
            if (string.atoi(self.root.get())<0 or
                string.atoi(self.root.get()) not in self.G.vertices):
                raise rootError
            self.result=[]
            self.result.append(string.atoi(self.root.get()))
            self.result.append(self.direction.get())
            return self.result
        except:
            showwarning("Warning", 
                        "Invalid root !!!\n"
                        "Please try again !")
            return 0
            
def BFSTreeCoords(G, root, direction, rect=None):
    BFSdistance = BFS(G,root,direction)[0]
    maxDistance=0
    maxBreadth=0
    if rect is None:
        rect = Rect()    
    list = {}
    for v in G.vertices:
        list[BFSdistance[v]] = []
    for v in G.vertices:
        list[BFSdistance[v]].append(v)
    maxDistance=len(list)
    for d in list.values():
        if len(d)>maxBreadth: maxBreadth=len(d)
    if maxDistance > 1:
        xDist=rect.height()/(maxDistance-1)
    else:
        xDist=0
    if maxBreadth > 1:
        yDist=rect.width()/(maxBreadth-1)
    else:
        yDist=0
    Coord1=rect.x2
    mid_y, dummy = rect.midpoint()
    
    G.xCoord={}
    G.yCoord={}
    for d in list.values():
        Coord2=mid_y-(len(d)-1)*yDist/2
        for v in d:
            G.xCoord[v]=Coord1+random.randint(-20,20)
            G.yCoord[v]=Coord2
            Coord2=Coord2+yDist 
        Coord1=Coord1-xDist
    return 1
    
    
class BFSTreeEmbedder(Embedder):

    def Name(self):
        return "BFS-Tree Layout"
        
    def Embed(self, theGraphEditor):
        if theGraphEditor.G.Order()==0:
            return
            
        theGraphEditor.config(cursor="watch")
        
        dial = BFSLayoutDialog(theGraphEditor)
        if dial.result is None: 
            theGraphEditor.config(cursor="")
            return	
            
        if BFSTreeCoords(theGraphEditor.G, dial.result[0], dial.result[1]):
            RedrawGraph(theGraphEditor)
            theGraphEditor.dirty = 1
                                        
        theGraphEditor.config(cursor="")
        
        
from math import *
from DataStructures import Queue

def RadialTreeBFS(G,root,direction='forward'):
    """ Calculate BFS distances and predecessor without showing animations.
        Also compute angles for a radial layout
        If G is directed, direction does matter:
    
        - 'forward'  BFS will use outgoing edges
        - 'backward' BFS will use incoming edges
    
        It uses gInfinity (from GatoGlobals.py) as infinite distance.
        returns (dist,pred) """
    
    
    Q = Queue()
    d = {}
    pred = {}
    angle = {}
    childrenrange = {}
    
    
    for v in G.vertices:
        d[v] = gInfinity
    d[root] = 0
    pred[root] = None
    angle[root] = 0
    childrenrange[root] = (0, 2 * pi)
    
    Q.Append(root)
    
    while Q.IsNotEmpty():
        v = Q.Top()
        if G.QDirected() == 1 and direction == 'backward':
            nbh = G.InNeighbors(v)
        else:
            nbh = G.Neighborhood(v)
            
            
            # Compute size of unseen Nbh
        unseen = 0 
        for w in nbh:
            if d[w] == gInfinity:
                unseen += 1
                
        if unseen > 0:
            range = childrenrange[v][1] - childrenrange[v][0]
            delta = range / float(unseen)
            delta2 = delta * 0.5
            left = childrenrange[v][0] + delta2
            
            for w in nbh:
                if d[w] == gInfinity:
                    angle[w] = left + delta2
                    childrenrange[w] = (left,left+delta)
                    left += delta
                    d[w] = d[v] + 1
                    #print (v,w), "angle = ", angle[w]," range = ", childrenrange[w]
                    Q.Append(w)
                    
    return (d,pred,angle)
    
    
def RadialToXY(degree, r, offset):
    return (r*sin(degree) + offset[0], r*cos(degree) + offset[1])
    
    
def BFSRadialTreeCoords(G, root, direction, rect=None):
    (BFSdistance,pred,angle) = RadialTreeBFS(G,root,direction)
    maxdist = max(max(BFSdistance.values()),1)
    
    G.xCoord={}
    G.yCoord={}
    if rect is None:
        rect = Rect()
    offset = rect.midpoint()    
    d = (min(rect.width(), rect.height())/2 - 50) / maxdist
    for v in G.vertices:
        try:
            (G.xCoord[v], G.yCoord[v]) = RadialToXY(angle[v], BFSdistance[v] * d, offset)
        except:
            return 0
    return 1
    
    
    
class BFSRadialTreeEmbedder(Embedder):

    def Name(self):
        return "BFS-Radial Tree Layout"
        
    def Embed(self, theGraphEditor):
        if theGraphEditor.G.Order()==0:
            return
            
        theGraphEditor.config(cursor="watch")
        
        dial = BFSLayoutDialog(theGraphEditor)
        if dial.result is None: 
            theGraphEditor.config(cursor="")
            return	
            
        if BFSRadialTreeCoords(theGraphEditor.G, dial.result[0], dial.result[1]):
            RedrawGraph(theGraphEditor)
            theGraphEditor.dirty = 1

        theGraphEditor.config(cursor="")
        
        
        
        
        
        
        
        #----------------------------------------------------------------------
        
""" Here instantiate all the embedders you want to make available to
    a client. """
embedder = [RandomEmbedder(),
            CircularEmbedder(),
            FPP_PlanarEmbedder(),
            Schnyder_PlanarEmbedder(),
            TreeEmbedder(),
            BFSTreeEmbedder(),
            BFSRadialTreeEmbedder()]



#!/usr/bin/env python
################################################################################
#
#       This file is part of Gato (Graph Animation Toolbox) 
#
#	file:   GatoTest.py
#	author: Alexander Schliep (alexander@schliep.org)
#
#       Copyright (C) 1998-2015, Alexander Schliep, Winfried Hochstaettler and 
#       Copyright 1998-2001 ZAIK/ZPR, Universitaet zu Koeln
#                                   
#       Contact: alexander@schliep.org, winfried.hochstaettler@fernuni-hagen.de             
#
#       Information: http://gato.sf.net
#
#       This library is free software; you can redistribute it and/or
#       modify it under the terms of the GNU Library General Public
#       License as published by the Free Software Foundation; either
#       version 2 of the License, or (at your option) any later version.
#
#       This library is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#       Library General Public License for more details.
#
#       You should have received a copy of the GNU Library General Public
#       License along with this library; if not, write to the Free
#       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#
#       This file is version $Revision: 670 $ 
#                       from $Date: 2015-01-13 16:04:11 -0500 (Tue, 13 Jan 2015) $
#             last change by $Author: schliep $.
#
################################################################################
import getopt
import sys
import os
import re
import fnmatch
import logging
from Gato import *
import GatoGlobals
g = GatoGlobals.AnimationParameters

#
# Negative cycle property
#
# 05-ShortestPaths/BellmanFord.alg === 05-ShortestPaths/11x11neg.cat
# 05-ShortestPaths/BellmanFord.alg === 05-ShortestPaths/NegCircuit.cat
# 05-ShortestPaths/BellmanFord.alg === 05-ShortestPaths/NegCircuit2.cat
# 05-ShortestPaths/DijkstraPQ.alg === 05-ShortestPaths/11x11neg.cat
# 05-ShortestPaths/DijkstraPQ.alg === 05-ShortestPaths/NegCircuit2.cat
# 05-ShortestPaths/FindPath.alg === 05-ShortestPaths/11x11neg.cat
#
#
# 07-MinimumCostFlows/SuccessiveShortestPath.alg === 06-MaximalFlows/Didacta.cat

def allInstances(exclude_algorithms=[], exclude_instances = {}):
    """ Compute instances for all *.alg and all *.cat files across directories.
        Algorithms need to set self.NeededProperties() in Prolog correctly to
        avoid incompatible algorithm-graph pairings. E.g.
        self.NeededProperties({'EvenOrder':1}) for WeightedMatching
    """
    testPath = "../CATBox/"
    directories = [
        '02-GraphsNetworks/',
        '03-MinimalSpanningTrees/',
        '04-LPDuality/',
        '05-ShortestPaths/',
        '06-MaximalFlows/',
        '07-MinimumCostFlows/',
        '08-Matching/',
        '09-WeightedMatching/'
        ]

    algorithms = []
    graphs = []

    for d in directories:
        algorithms += [os.path.join(d,file) for file in os.listdir(os.path.join(testPath,d)) \
                       if (fnmatch.fnmatch(file, '*.alg') and not file in exclude_algorithms)]
        graphs += [os.path.join(d,file) for file in os.listdir(os.path.join(testPath,d)) if \
                   fnmatch.fnmatch(file, '*.cat')]

    instance = {}
    for a in algorithms:
        if a in exclude_instances:
            instance[a] = list(set(graphs) - set(exclude_instances[a]))
        else:
            instance[a] = graphs

    return instance


# instance is a dictionary where the keys are 
# algorithms and their value is a list of graphs
# they will work on.
#
# Testcases are all combinations of algorithm with all their
# graphs.
instance = {
    '02-GraphsNetworks/BFS-components.alg':[
    '02-GraphsNetworks/3Components.cat'
    ],
    '02-GraphsNetworks/BFS.alg':[
    '02-GraphsNetworks/DoubleTriangle.cat'
    ],
    '02-GraphsNetworks/BFStoDFS.alg':[
    '02-GraphsNetworks/BFS.cat'
    ],
    '02-GraphsNetworks/DFS-Recursive.alg':[
    '02-GraphsNetworks/DoubleTriangle.cat'
    ],
    '02-GraphsNetworks/DFS.alg':[
    '02-GraphsNetworks/DoubleTriangle.cat'
    ],
    '03-MinimalSpanningTrees/Kruskal.alg':[
    '03-MinimalSpanningTrees/Kruskal4.cat'
    ],
    '03-MinimalSpanningTrees/KruskalFindCircuit.alg':[
    '03-MinimalSpanningTrees/Kruskal4.cat'
    ],
    '03-MinimalSpanningTrees/KruskalInefficient.alg':[
    '03-MinimalSpanningTrees/Kruskal4.cat'
    ],
    '03-MinimalSpanningTrees/KruskalTrace.alg':[
    '03-MinimalSpanningTrees/Kruskal4.cat'
    ],
    # XXX Not sure whether it terminates: on a small instance it does
    # But since this is random this is not good for regression testing
    '03-MinimalSpanningTrees/MSTInteractive.alg':[
    '03-MinimalSpanningTrees/Kruskal3.cat'
    ],
    '03-MinimalSpanningTrees/MatroidDualKruskal.alg':[
    '03-MinimalSpanningTrees/Kruskal3.cat'    
    ],
    '03-MinimalSpanningTrees/Prim.alg':[
    '03-MinimalSpanningTrees/Prim1.cat'
    ],
    '04-LPDuality/PrimalDualKruskal.alg':[
    '04-LPDuality/PD_Kruskal3.cat'
    ],
    '05-ShortestPaths/BellmanFord.alg':[
    '05-ShortestPaths/BellmanFordWC.cat'
    ],
    '05-ShortestPaths/Dijkstra.alg':[
    '05-ShortestPaths/DijkstraRelabel.cat'
    ],
    '05-ShortestPaths/DijkstraPQ.alg':[
    '05-ShortestPaths/DijkstraRelabel.cat'
    ],
    '05-ShortestPaths/FindPath.alg':[
    '05-ShortestPaths/ShortestPathsUndirected05.cat'
    ],
    '05-ShortestPaths/FindPathEuclid.alg':[
    '05-ShortestPaths/ShortestPathsUndirected05.cat'
    ],
    '05-ShortestPaths/NegativeCircuits.alg':[
    '05-ShortestPaths/NegCircuit.cat'
    ],
    '05-ShortestPaths/TwoSources.alg':[
    '05-ShortestPaths/ShortestPathsUndirected05.cat'    
    ],
    '06-MaximalFlows/FordFulkerson.alg':[
    '06-MaximalFlows/FordFulkerson4.cat'
    ],
    '06-MaximalFlows/PreflowPush.alg':[
    '06-MaximalFlows/FordFulkerson4.cat'
    ],
    '07-MinimumCostFlows/CostScaling.alg':[
    '07-MinimumCostFlows/MCF4to1B.cat'
    ],
    '07-MinimumCostFlows/NegativeCircuit.alg':[
    '07-MinimumCostFlows/MCFCycle.cat'
    ],
    '07-MinimumCostFlows/SuccessiveShortestPath.alg':[
    '07-MinimumCostFlows/MCF4to1B.cat'    
    ],
    '08-Matching/Bipartite.alg':[
    '08-Matching/Bi006.cat'
    ],
    '08-Matching/CardinalityMatching.alg':[
    '08-Matching/Edmonds5.cat'
    ],
    '09-WeightedMatching/WeightedMatching.alg':[
    '09-WeightedMatching/rote4.cat'
    ]
    }


# Production version:
#
# List of algorithms and instances actually used in the book
# and some extra instances
svg_instance = [
    {
        'chapter_directory': '02-GraphsNetworks',
        'chapter_number': 2,
        'title': 'Basics, Notation and Data Structures',
        'algorithms': [
            {
                'title': 'BFS-components',
                'file': 'BFS-components.alg',
                'description': 'Finds connected components in a graph using iterated ' \
                'Breadth-First-Search (BFS).',
                'graphs': [
                    '02-GraphsNetworks/1Component.cat', '02-GraphsNetworks/2Components.cat',
                    '02-GraphsNetworks/3Components.cat'
                ]
            },
            {
                'title': 'DFS Recursive',
                'file': 'DFS-Recursive.alg',
                'description': 'A recursive implementation of Depth-First-Search (DFS), ' \
                'which computes a DFS labeling',
                'graphs': [
                    '02-GraphsNetworks/BFS.cat', '02-GraphsNetworks/3Components.cat'
                ]
            },
            {
                'title': 'Iterative BFS',
                'file': 'BFS.alg',
                'description': 'An iterative implementation of Breadth-First-Search ' \
                '(BFS), which computes a BFS labeling',
                'graphs': [
                    '02-GraphsNetworks/BFS.cat', '02-GraphsNetworks/3Components.cat'
                ]
            },
            {
                'title': 'BFS to DFS',
                'file': 'BFStoDFS.alg',
                'description': 'Converting the iterative BFS implementation to an iterative ' \
                'DFS implementation by exchanging the Queue by a Stack.',
                'graphs': [
                    '02-GraphsNetworks/BFS.cat', '06-MaximalFlows/FordFulkerson5.cat'
                ]
            },
            {
                'title': 'Iterative DFS',
                'file': 'DFS.alg',
                'description': 'An iterative implementation of Depth-First-Search (DFS), which ' \
                'computes a DFS labeling',
                'graphs': [
                    '02-GraphsNetworks/BFS.cat', '02-GraphsNetworks/3Components.cat'
                ]
            }
        ]
    },
    {
        'chapter_directory': '03-MinimalSpanningTrees',
        'chapter_number': 3,
        'title': 'Minimum Spanning Trees',
        'algorithms': [
            {   
                'title': 'Kruskal\'s Algorithm',
                'file': 'Kruskal.alg',
                'description': 'Kruskal\'s algorithm computes a minimum spanning tree in ' \
                'a connected, weighted graph.',
                'graphs': [
                    '03-MinimalSpanningTrees/Kruskal1.cat', '03-MinimalSpanningTrees/Kruskal2.cat',
                    '03-MinimalSpanningTrees/Kruskal3.cat',
                    '03-MinimalSpanningTrees/Prim1.cat','03-MinimalSpanningTrees/Prim2.cat'
                ]
            },
            {
                'title': 'Kruskal\'s Algorithm using FindCircuit',
                'file': 'KruskalFindCircuit.alg',
                'description': 'A more detailled, but naive implementation of Kruskal\'s '\
                'algorithm, which checks for each edge to be added whether it completes a circuit.',
                'graphs': [
                    '03-MinimalSpanningTrees/Kruskal1.cat','03-MinimalSpanningTrees/Kruskal2.cat',
                    '03-MinimalSpanningTrees/Kruskal3.cat',
                    '03-MinimalSpanningTrees/Prim1.cat','03-MinimalSpanningTrees/Prim2.cat'
                ]
            },
            {
                'title': 'Inefficient Kruskal\'s Algorithm',
                'file': 'KruskalInefficient.alg',
                'description': 'An improved, but still somewhat naive implementation of ' \
                'Kruskal\'s algorithm, which bypasses the explicit test for circuit ' \
                'completion by maintaining component labels for vertices and testing for equality.',
                'graphs': [
                    '03-MinimalSpanningTrees/Kruskal1.cat','03-MinimalSpanningTrees/Kruskal2.cat',
                    '03-MinimalSpanningTrees/Kruskal3.cat',
                    '03-MinimalSpanningTrees/Prim1.cat','03-MinimalSpanningTrees/Prim2.cat'
                ]
            },
            {
                'title': 'Efficient Kruskal\'s Algorithm',
                'file': 'KruskalTrace.alg',
                'description': 'An efficient implementation of Kruskal\'s algorithm, which ' \
                'minimizes the number of component label updates.',
                'graphs': [
                    '03-MinimalSpanningTrees/Kruskal1.cat','03-MinimalSpanningTrees/Kruskal2.cat',
                    '03-MinimalSpanningTrees/Kruskal3.cat',
                    '03-MinimalSpanningTrees/Prim1.cat','03-MinimalSpanningTrees/Prim2.cat'
                ]   
            },
            {
                'title': 'Prim\'s Algorithm',
                'file': 'Prim.alg',
                'description': 'Prim\'s Algorithm computes a minimum spanning tree in a ' \
                'connected, weighted graph.',
                'graphs': [
                    '03-MinimalSpanningTrees/Kruskal1.cat','03-MinimalSpanningTrees/Kruskal2.cat',
                    '03-MinimalSpanningTrees/Kruskal3.cat',
                    '03-MinimalSpanningTrees/Prim1.cat','03-MinimalSpanningTrees/Prim2.cat'
                ]
            },
            {
                'title': 'Matroid Dual of Kruskal\'s Algorithm',
                'file': 'MatroidDualKruskal.alg',
                'description': 'An implementation of a matroid dual of Kruskal\'s Algorithm. ', 
                'graphs': [
                    '03-MinimalSpanningTrees/Kruskal1.cat','03-MinimalSpanningTrees/Kruskal2.cat',
                    '03-MinimalSpanningTrees/Kruskal3.cat',
                    '03-MinimalSpanningTrees/Prim1.cat','03-MinimalSpanningTrees/Prim2.cat'
                ]
            },
        ]
    },
    {
        'chapter_directory': '04-LPDuality',
        'chapter_number': 4,
        'title': 'Linear Programming Duality',
        'algorithms': [
            {
                'title': 'Primal Dual of Kruskal\'s Algorithm',
                'file': 'PrimalDualKruskal.alg',
                'description': 'A primal dual version of Kruskal\'s Algorithm.',
                'graphs': [
                    '04-LPDuality/PD_Kruskal1.cat','04-LPDuality/PD_Kruskal2.cat',
                    '04-LPDuality/PD_Kruskal3.cat', '04-LPDuality/PD_Kruskal4.cat',
                    '04-LPDuality/PD_Kruskal5.cat'
                ]
            }
        ]
    },
    {
        'chapter_directory': '05-ShortestPaths',
        'chapter_number': 5,
        'title': 'Shortest Paths',
        'algorithms': [
            {
                'title': 'Dijkstra\'s algorithm',
                'file': 'Dijkstra.alg',
                'description': 'Dijkstra\'s algorithm for finding a shortest path tree in ' \
                'a graph. This is an example of a label setting algorithm.',
                'graphs': [
                    '05-ShortestPaths/ShortestPathsUndirected02.cat',
                    '05-ShortestPaths/ShortestPathsUndirected03.cat',
                    '05-ShortestPaths/7x7grid.cat',
                    '05-ShortestPaths/NegArc.cat',
                    '05-ShortestPaths/NegCircuit.cat'                   
                ]
            },
            {
                'title': 'Dijkstra\'s algorithm using a Priority Queue',
                'file': 'DijkstraPQ.alg',
                'description': 'Dijkstra\'s algorithm for finding a shortest path tree in a' \
                'graph implemented using a Priority Queue.',
                'graphs': [
                    '05-ShortestPaths/ShortestPathsUndirected02.cat',
                    '05-ShortestPaths/ShortestPathsUndirected03.cat',
                    '05-ShortestPaths/7x7grid.cat',      
                    '05-ShortestPaths/NegArc.cat',
                    '05-ShortestPaths/NegCircuit.cat'                   
                ]
            },
            {
                'title': 'Find Path',
                'file': 'FindPath.alg',
                'description': 'A variant of Dijkstra\'s algorithm for finding a shortest s-t ' \
                'path in a graph. Terminates early, as soon as a shortest s-t path has been found.',
                'graphs': [
                    '05-ShortestPaths/ShortestPathsUndirected03.cat',
                    '05-ShortestPaths/ShortestPathsUndirected06.cat',
                    '05-ShortestPaths/7x7grid.cat'
                ]
            },
            {
                'title': 'Find Path in an Euclidean Graph',
                'file': 'FindPathEuclid.alg',
                'description': 'A variant of Dijkstra\'s algorithm for finding a shortest s-t ' \
                'path in an Euclidean graph (edge weights correspond to Euclidean distance ' \
                'between vertices). Terminates early, as soon as a shortest s-t path has been ' \
                'found and only visits a small part of the graph. Similar ideas are used in '\
                'A*-type algorithms.',
                'graphs': [
                    '05-ShortestPaths/ShortestPathsUndirected03.cat',
                    '05-ShortestPaths/ShortestPathsUndirected06.cat',
                    '05-ShortestPaths/7x7grid.cat'
                 ]
            },
            {
                'title': 'Bellman Ford',
                'file': 'BellmanFord.alg',
                'description': 'Bellman and Ford\'s algorithm for finding a shortest path tree in ' \
                'a graph with a label correcting approach.',
                'graphs': [
                    '05-ShortestPaths/ShortestPathsUndirected02.cat',
                    '05-ShortestPaths/ShortestPathsUndirected03.cat',
                    '05-ShortestPaths/7x7grid.cat',
                    '05-ShortestPaths/BellmanFordWC.cat',
                    '05-ShortestPaths/NegArc.cat'
                ]
            },
            {
                'title': 'Finding Negative Circuits',
                'file': 'NegativeCircuits.alg',
                'description': 'Most shortest path algorithms require that graphs do not have '\
                    'negative circuits. This algorithms detects negative circuits in a graph.',
                'graphs': [
                    '05-ShortestPaths/NegCircuit.cat',
                    '05-ShortestPaths/NegCircuit2.cat'
                ]
            },
            # Could use highlighting of paths on this
            {
                'title': 'Find Path from Two Sources',
                'file': 'TwoSources.alg',
                'description': 'Finding a path can be accelerated by searching from start and' \
                'destination simultaneously.',
                'graphs': [
                    '05-ShortestPaths/ShortestPathsUndirected06.cat'
                ]
            }
        ]
    },
    {
        'chapter_directory': '06-MaximalFlows',
        'chapter_number': 6,
        'title': 'Maximal flows in capacitated networks',
        'algorithms': [
            {
                'title': 'The Ford Fulkerson algorithm',
                'file': 'FordFulkerson.alg',
                'description': 'The Ford-Fulkerson algorithm finds a maximal flow in a ' \
                'capacitated network by successive shortest path applications.',
                'graphs': [
                    '06-MaximalFlows/FordFulkerson1.cat',
                    '06-MaximalFlows/FordFulkerson4.cat',
                    '06-MaximalFlows/FordFulkerson6.cat',
                    '06-MaximalFlows/FordFulkersonBad.cat',
                    '06-MaximalFlows/FordFulkersonWC.cat',
                    '06-MaximalFlows/PreflowPushWC.cat',
                    '06-MaximalFlows/EdmondsKarpWC.cat'
                ]
            },
            {
                'title': 'Preflow Push',
                'file': 'PreflowPush.alg',
                'description': 'The Preflow Push algorithm due to Goldberg and Tarjan ' \
                'computes a maximal flow by saturating a network and correcting possible ' \
                'excesses later',
                'graphs': [
                    '06-MaximalFlows/PreflowPush1.cat',
                    '06-MaximalFlows/PreflowPush6.cat',
                    '06-MaximalFlows/FordFulkerson1.cat',
                    '06-MaximalFlows/FordFulkerson4.cat',
                    '06-MaximalFlows/FordFulkerson6.cat',
                    '06-MaximalFlows/FordFulkersonBad.cat',
                    '06-MaximalFlows/FordFulkersonWC.cat',
                    '06-MaximalFlows/PreflowPushWC.cat'
                ]
            }
        ]
    },
    {
        'chapter_directory': '07-MinimumCostFlows',
        'chapter_number': 7,
        'title': 'Minimum-cost Flows',
        'algorithms': [
            {
                'title': 'Negative Circuits',
                'file': 'NegativeCircuit.alg',
                'description': 'The Negative Circuits algorithms establishes a maximal flow ' \
                'and reduces costs by finding negative circuits.',
                'graphs': [
                    '07-MinimumCostFlows/MCF4to1A.cat', '07-MinimumCostFlows/MCFCycle.cat'
                ]
            },
            {
                'title': 'Successive Shortest Paths',
                'file': 'SuccessiveShortestPath.alg',
                'description': 'This algorithm finds a minimum cost flow by  repeatedly ' \
                ' finding shortest, augmenting paths ',
                'graphs': [
                    '07-MinimumCostFlows/MCF4to1A.cat', '07-MinimumCostFlows/MCFCycle.cat'
                ]
            },
            {
                'title': 'Cost Scaling',
                'file': 'CostScaling.alg',
                'description': 'The cost scaling algorithm directly operates on the bits ' \
                'of the cost function',
                'graphs': [
                    '07-MinimumCostFlows/MCF4to1A.cat', '07-MinimumCostFlows/MCFCycle.cat'
                ]
            },
        ]
    },

    {
        'chapter_directory': '08-Matching',
        'chapter_number': 8,
        'title': 'Matching',
        'algorithms': [
            {
                'title': 'Bipartite',
                'file': 'Bipartite.alg',
                'description': 'This algorithm finds a maximal cardinality matching' \
                'in a bipartite graph. In other words it maximes the number of matched ' \
                'vertices',
                'graphs': [
                    '08-Matching/Bi003.cat', '08-Matching/Bi006.cat', '08-Matching/Bi008.cat',
                    '08-Matching/Bi009.cat', '08-Matching/Bi010.cat'
                ]
            },
            {
                'title': 'Cardinality Matching',
                'file': 'CardinalityMatching.alg',
                'description': 'This algorithm finds a maximal cardinality matching in graphs ' \
                'which are not necessarily bipartite.',
                'graphs': [
                    '08-Matching/Edmonds1.cat', '08-Matching/Edmonds3.cat',
                    '08-Matching/Edmonds6.cat', '08-Matching/Edmonds8.cat', '08-Matching/Koenig.cat'
                ]
            }
        ]
    },

    {
        'chapter_directory': '09-WeightedMatching',
        'chapter_number': 9,
        'title': 'Weighted Matching',
        'algorithms': [
            {
                'title': 'Weighted Matching',
                'file': 'WeightedMatching.alg',
                'description': 'This algorithm considers vertices as points in 2-dimensional ' \
                'space and implicitely assumes the existence of all edges in the complete ' \
                'graph. Furthermore, edge weights are Euclidean. It finds a matching of minimal ' \
                'weight. The matching is perfect if the number of vertices is even', 
                'graphs': [
                    '09-WeightedMatching/twotriangles.cat', '09-WeightedMatching/3223.cat',
                    '09-WeightedMatching/11vs13.cat'
                ]
            }
        ]
    }
]


#
# Dictionary of graph name substitutes(e.g. if a graph name is too long we can map it to a different name here, and the sub will be used in the index page)
#
friendly_graph_names = {
    'ShortestPathsUndirected01': 'Shortest Paths Undirected 01',
    'ShortestPathsUndirected02': 'Shortest Paths Undirected 02',
    'ShortestPathsUndirected03': 'Shortest Paths Undirected 03',
    'ShortestPathsUndirected04': 'Shortest Paths Undirected 04',
    'ShortestPathsUndirected05': 'Shortest Paths Undirected 05',
    'ShortestPathsUndirected06': 'Shortest Paths Undirected 06',
}

# Keys: e.g. '09-WeightedMatching/k4.cat', values: graph description
graph_descriptions = {
    '02-GraphsNetworks/BFS.cat': 'BFS Graph Description'
}

def create_svg_index_page(graph_pngs):
    ''' Creates an HTML index page that leads to the SVG 
        animations generated from the svg_instance dictionary
    '''
    import jinja2
    svg_locations = {}
    for chapter_dict in svg_instance:
        for algo in chapter_dict['algorithms']:
            algo_name = os.path.splitext(algo['file'])[0]
            algo['algo_div_id'] = algo['title'].replace(' ', '').replace('-', '')
            for graph in algo['graphs']:
                graph_name = os.path.splitext(os.path.basename(graph))[0]
                svg_locations[algo['title'] + graph] = algo_name + '--' + graph_name + '.html'

    with open('svg_index_template.html', 'r') as template_in:
        with open('svgs/index.html', 'w') as index_out:
            template = jinja2.Template(template_in.read())
            index_out.write(template.render({
                'svg_instance': svg_instance, 
                'graph_pngs': graph_pngs, 
                'graph_descriptions': graph_descriptions,
                'svg_locations': svg_locations
             }))

def should_generate_animation(algo_file_name, graph_file_name, svg_file_name):
    # If the svg we have is more current than all it's component files then we don't
    # have to generate the animation
    catbox_path = os.path.split(algo_file_name)[0]
    algo_pro_file_name = os.path.splitext(os.path.basename(algo_file_name))[0] + '.pro'
    if os.path.isfile(svg_file_name):
        svg_age = os.path.getmtime(svg_file_name)
    else: # File does not exist, so generate 
        return True
    graph_age = os.path.getmtime(graph_file_name)
    pro_age = os.path.getmtime(os.path.join(catbox_path, algo_pro_file_name))
    algo_age = os.path.getmtime(algo_file_name)
    webgato_js_age = os.path.getmtime('WebGatoJS.py')

    if svg_age > max(graph_age, pro_age, algo_age, webgato_js_age):
        return False
    return True

def should_generate_png(algo_file_name, graph_file_name, png_file_name):
    # If the png we have is more current than the graph/algo combo it came from then we don't have to re-generate
    catbox_path = os.path.split(algo_file_name)[0]
    algo_pro_file_name = os.path.splitext(os.path.basename(algo_file_name))[0] + '.pro'
    if os.path.isfile(png_file_name):
        png_age = os.path.getmtime(png_file_name)
    else: # File does not exist, so generate 
        return True
    graph_age = os.path.getmtime(graph_file_name)
    pro_age = os.path.getmtime(os.path.join(catbox_path, algo_pro_file_name))
    algo_age = os.path.getmtime(algo_file_name)

    if png_age > max(graph_age, pro_age, algo_age):
        return False
    return True

#------------------------------------------------------------------
def usage():
    print "Usage: GatoTest.py"
    print "       GatoTest.py -v -d"
    print "               -v or --verbose switches on the logging information"
    print "               -d or --debug switches on the debugging  information"
    print "               -s or --svg generates svg animations if the animation sources have been updated"
    print "               -f or --force_animation used with -s to force generation of svg animations, even if there have been no updates"



if __name__ == '__main__':
    try:
        opts, args = getopt.getopt(sys.argv[1:], "asdvtf",
                                   ["all", "svg","debug","verbose", "test", "force_animation"])
    except getopt.GetoptError:
        usage()
        exit()

    print "Welcome to GatoTest!"

    all = False
    svg = False
    force_animation = False
    debug = False
    verbose = False
    test = False
    for o, a in opts:
        if o in ("-a", "--all"):
            all = True
        if o in ("-v", "--verbose"):
            verbose = True
        if o in ("-d", "--debug"):
            debug = True
        if o in ("-s", "--svg"):
            svg = True
        if o in ("-t", "--test"):
            test = True
        if o in ("-f", "--force_animation"):
            force_animation = True
            
            
    if test:
        testPath = "./"
        tests = [ ("BFS.alg", "sample.cat") ]
    elif not svg:
        if all:
            ei = {
                '05-ShortestPaths/BellmanFord.alg':['05-ShortestPaths/11x11neg.cat',
                                                    '05-ShortestPaths/NegCircuit.cat',
                                                    '05-ShortestPaths/NegCircuit2.cat'
                                                    ],
                '05-ShortestPaths/FindPath.alg':['05-ShortestPaths/11x11neg.cat',
                                                 '05-ShortestPaths/NegCircuit.cat',
                                                 '05-ShortestPaths/NegCircuit2.cat'
                                                 ],
                '05-ShortestPaths/FindPathEuclid.alg':['05-ShortestPaths/11x11neg.cat',
                                                       '05-ShortestPaths/NegCircuit.cat',
                                                       '05-ShortestPaths/NegCircuit2.cat'
                                                       ],
                '05-ShortestPaths/Dijkstra.alg':['05-ShortestPaths/11x11neg.cat',
                                                 '05-ShortestPaths/NegCircuit.cat',
                                                 '05-ShortestPaths/NegCircuit2.cat'
                                                 ],
                '05-ShortestPaths/DijkstraPQ.alg':['05-ShortestPaths/11x11neg.cat',
                                                   '05-ShortestPaths/NegCircuit.cat',
                                                   '05-ShortestPaths/NegCircuit2.cat'
                                                   ]
                }
            instance = allInstances(exclude_algorithms=['MSTInteractive.alg'],
                                    exclude_instances=ei)
        algorithms = instance.keys()
        algorithms.sort()
        tests = [(algo, graph) for algo in algorithms for graph in instance[algo]]
        testPath = "../CATBox/"


    # To speed up running of tests
    g.BlinkRepeat = 1 
    g.BlinkRate = 2
    
    g.Interactive = 0 # Same effect as hitting continue for interactive lines
    log = logging.getLogger("Gato")
    
    app = AlgoWin()
    if debug:
        app.algorithm.logAnimator = 2

    if verbose:
        if sys.version_info[0:2] < (2,4):
            log.addHandler(logging.StreamHandler(sys.stdout))
            log.setLevel(logging.DEBUG)
        else:
            logging.basicConfig(level=logging.DEBUG,
                                stream=sys.stdout,
                                format='%(name)s %(levelname)s %(message)s')
    else:
        if app.windowingsystem == 'win32':

           class NullHandler(logging.Handler):
               def emit(self, record):
                   pass
           h = NullHandler()
           logging.getLogger("Gato").addHandler(h)

        else:
            logging.basicConfig(level=logging.WARNING,
                                filename='/tmp/Gato.log',
                                filemode='w',
                                    format='%(name)s %(levelname)s %(message)s')        
    
    if svg:
        print "# Producing SVG output"
        if not os.path.exists('./svgs'):
            os.makedirs('./svgs')

        # Warn the user if they don't have the correct libraries for PNG generation
        has_png_libs = True
        try:
            import cairo
            import rsvg
        except:
            has_png_libs = False
            err_msg = ('*******\nWARNING:\n'
                'Generation of an SVG Index page requires the python packages for cairo(http://cairographics.org/)'
                ' and librsvg(https://wiki.gnome.org/action/show/Projects/LibRsvg?action=show&redirect=LibRsvg).'
                '  SVG generation will continue, but no index page will be generated.\n*******')
            print err_msg

        graph_pngs = {}
        testPath = "../CATBox/"
        for chapter_dict in svg_instance:
            for algo in chapter_dict['algorithms']:
                for graph_file in algo['graphs']:
                    log.info("=== TEST === "+algo['file']+" === "+graph_file+" ===")
                    print "=== TEST === "+algo['file']+" === "+graph_file+" ==="
                    graph_name = os.path.splitext(os.path.basename(graph_file))[0]
                    png_file_name = 'svgs/img/%s.png' % (graph_name)
                    svg_file_name = 'svgs/%s--%s.html' % (os.path.splitext(algo['file'])[0], graph_name)
                    algo_location = testPath + chapter_dict['chapter_directory'] + '/' + algo['file']
                    if should_generate_animation(algo_location, testPath+graph_file, svg_file_name) or force_animation:
                        app.OpenAlgorithm(algo_location)
                        g.Interactive = 0 # This is set to 0 above.  Do we need to do it here as well?
                        g.GeneratingSVG = 1
                        app.algorithm.ClearBreakpoints()
                        app.update_idletasks()
                        app.update()
                        app.OpenGraph(testPath + graph_file)
                        app.update_idletasks()
                        app.update()
                        # Run it ...
                        app.after_idle(app.CmdContinue) # after idle needed since CmdStart
                        # does not return
                        app.CmdStart()
                        app.update_idletasks()

                        # Generate the SVG
                        app.ExportSVGAnimation(svg_file_name, chapter_number=chapter_dict['chapter_number'], algo_div=algo['title'].replace(' ', '').replace('-',''))

                    index_graph_name = friendly_graph_names.get(graph_name, graph_name)
                    if should_generate_png(algo_location, testPath+graph_file, png_file_name):
                        # Generate the PNG
                        if graph_file not in graph_pngs and has_png_libs:
                            png_dimensions = app.ExportSVG(png_file_name, write_to_png=True)
                            path_from_index = '/'.join(png_file_name.split('/')[1:])
                            graph_pngs[graph_file] = {'file': path_from_index, 'name': index_graph_name, 'width': png_dimensions['width'], 'height': png_dimensions['height']}
                    else:
                        from PIL import Image
                        img = Image.open(png_file_name)
                        graph_pngs[graph_file] = {'file': '/'.join(png_file_name.split('/')[1:]), 'name': index_graph_name, 'width': img.size[0], 'height': img.size[1]}

        if has_png_libs:
            create_svg_index_page(graph_pngs)

    else:
        for case in tests:
            log.info("=== TEST === "+case[0]+" === "+case[1]+" ===")
            app.OpenAlgorithm(testPath + case[0])
            g.Interactive = 0
            app.algorithm.ClearBreakpoints()
            app.update_idletasks()
            app.update()
            app.OpenGraph(testPath + case[1])
            app.update_idletasks()
            app.update()
            # Run it ...
            app.after_idle(app.CmdContinue) # after idle needed since CmdStart
            # does not return
            app.CmdStart()
            app.update_idletasks()
            #app.mainloop()

            



################################################################################
#
#       This file is part of Gato (Graph Animation Toolbox) 
#
#	file:   GraphCreator.py
#	author: Ramazan Buzdemir (buzdemir@zpr.uni-koeln.de)
#
#       Copyright (C) 1998-2015, Alexander Schliep, Winfried Hochstaettler and 
#       Copyright 1998-2001 ZAIK/ZPR, Universitaet zu Koeln
#                                   
#       Contact: alexander@schliep.org, winfried.hochstaettler@fernuni-hagen.de             
#
#       Information: http://gato.sf.net
#
#       This library is free software; you can redistribute it and/or
#       modify it under the terms of the GNU Library General Public
#       License as published by the Free Software Foundation; either
#       version 2 of the License, or (at your option) any later version.
#
#       This library is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#       Library General Public License for more details.
#
#       You should have received a copy of the GNU Library General Public
#       License along with this library; if not, write to the Free
#       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#
#       This file is version $Revision: 670 $ 
#                       from $Date: 2015-01-13 16:04:11 -0500 (Tue, 13 Jan 2015) $
#             last change by $Author: schliep $.
#
################################################################################

from Graph import *
from Embedder import *
import random
from tkMessageBox import askokcancel

class Creator:
    """ This class provides an abstract Creator as
        a base for actual Creator implementations """
    
    def Name(self):
        """ Return a short descriptive name for the creator e.g. usable as 
            a menu item """
        return "none"
        
    def Create(self, theGraphEditor):
        """ Create a new graph. """
        return none

    def CheckDirtyAndCreate(self, theGraphEditor):
        """ Create a new graph. """

        if theGraphEditor.dirty == 1:
            if not askokcancel("Open Graph",
                               "Graph changed since last saved. Do you want to overwrite it?"):
                return            
        self.Create(theGraphEditor)
        
        
def DrawNewGraph(theGraphEditor,G,direction):

    theGraphEditor.dirty = 0
    theGraphEditor.NewGraph(direction,1,0,'None',0,'One',0)
    
    for v in G.vertices:
        theGraphEditor.AddVertex(G.xCoord[v],G.yCoord[v])
        
    for e in G.Edges():
        theGraphEditor.AddEdge(e[0],e[1])
    theGraphEditor.dirty = 1
    
        
#----------------------------------------------------------------------
from Tkinter import *
import tkSimpleDialog 
import string
from tkMessageBox import showwarning

class Dialog(tkSimpleDialog.Dialog):

    def __init__(self, master, planar, visible, Text):
        self.planar=planar
        self.visible=visible
        tkSimpleDialog.Dialog.__init__(self, master, Text)
        
        
    def body(self, master):
        self.resizable(0,0)
        
        self.number_of_nodes=StringVar()
        self.number_of_nodes.set("1")
        label = Label(master, text="number of nodes :", anchor=W)
        label.grid(row=0, column=0, padx=0, pady=2, sticky="w")
        entry=Entry(master, width=6, exportselection=FALSE,
                    textvariable=self.number_of_nodes)
        entry.selection_range(0,1)
        entry.focus_set()
        entry.grid(row=0,column=1, padx=2, pady=2, sticky="w")
        
        self.number_of_edges=StringVar()
        self.number_of_edges.set("0")
        if self.visible:
            label = Label(master, text="number of edges :", anchor=W)
            label.grid(row=1, column=0, padx=0, pady=2, sticky="w")
            entry=Entry(master, width=6, exportselection=FALSE,
                        textvariable=self.number_of_edges)
            entry.selection_range(0,1)
            entry.focus_set()
            entry.grid(row=1,column=1, padx=2, pady=2, sticky="w")
            
        self.direction=IntVar()
        self.direction.set(0)
        radio=Radiobutton(master, text="Undirected", variable=self.direction,
                          value=0)
        radio.grid(row=0, column=2, padx=2, pady=2, sticky="w") 
        radio=Radiobutton(master, text="Directed", variable=self.direction,
                          value=1)
        radio.grid(row=1, column=2, padx=2, pady=2, sticky="w")
        
        label = Label(master, text=" ")
        label.grid(row=0, column=3, padx=5, pady=2) 
        
        self.layout=IntVar()
        self.layout.set(0)
        radio=Radiobutton(master, text="Randomize Layout",
                          variable=self.layout, value=0,
                          width=23, indicatoron=0, selectcolor="white")
        radio.grid(row=0, column=4, padx=3, pady=2, sticky="w")  
        radio=Radiobutton(master, text="Circular Layout",
                          variable=self.layout, value=1,
                          width=23, indicatoron=0, selectcolor="white")
        radio.grid(row=1, column=4, padx=3, pady=2, sticky="w") 
        if self.planar:
            radio=Radiobutton(master, text="Planar Layout (FPP)",
                              variable=self.layout, value=2,
                              width=23, indicatoron=0, selectcolor="white")
            radio.grid(row=3, column=4, padx=3, pady=2, sticky="w") 
            radio=Radiobutton(master, text="Planar Layout (Schnyder)",
                              variable=self.layout, value=3,
                              width=23, indicatoron=0, selectcolor="white")
            radio.grid(row=4, column=4, padx=3, pady=2, sticky="w")
            
    def validate(self):
        try:
            n=string.atoi(self.number_of_nodes.get())
            if n<1 or n>200:
                raise nodeError
        except:
            showwarning("Please try again !",
                        "min. number of nodes = 1\n" 
                        "max. number of nodes = 200")
            return 0            
            
        try:
            m=string.atoi(self.number_of_edges.get())
            
            if self.planar:
                if n==1: max_m=0
                else: max_m=6*n-12
            else:
                max_m=n*n-n
                
            if self.direction.get()==0:
                max_m = max_m/2
                
            if m<0 or m>max_m:
                raise edgeError
                
        except:
            showwarning("Please try again !",
                        "min. number of edges = 0\n"
                        "max. number of edges = %i" %max_m)
            return 0
            
        self.result=[]
        self.result.append(n)
        self.result.append(m)
        self.result.append(self.direction.get())
        self.result.append(self.layout.get())
        return self.result
        
        #----------------------------------------------------------------------
def CompleteEdges(G,n,direction):
    Edges=[]
    for i in range(0,n):
        source=G.vertices[i]
        for j in range(i+1,n):   
            target=G.vertices[j]
            Edges.append((source,target))
            if direction==1: Edges.append((target,source))
    return Edges
    
def MaximalPlanarEdges(G,n,direction):
    Edges=[] #6*n
    
    AdjEdges={}
    for v in G.vertices:
        AdjEdges[v]=[]
        
    index=0
    a=G.vertices[index]
    index=index+1
    b=G.vertices[index]
    index=index+1
    
    Edges.append((a,b))
    AdjEdges[a].append((a,b))
    Edges.append((b,a))
    AdjEdges[b].append((b,a))
    
    m=2
    while index < n:
        e=Edges[random.randint(0,m-1)]
        v=G.vertices[index]
        index=index+1
        
        while e[1]!=v:
            x=(v,e[0])
            Edges.append(x)
            m=m+1
            AdjEdges[v].append(x)
            
            y=(e[0],v)
            Edges.append(y)
            m=m+1
            AdjEdges[e[0]].insert(AdjEdges[e[0]].index(e)+1,y)
            
            index2=AdjEdges[e[1]].index((e[1],e[0]))
            if index2==0:
                e=AdjEdges[e[1]][-1]
            else:
                e=AdjEdges[e[1]][index2-1]
                
    if direction==0: # undirected
        m=m-1
        while m>0:
            del Edges[m]
            m=m-2
            
    return Edges
    
    #----------------------------------------------------------------------
class completeGraphCreator(Creator):

    def Name(self):
        return "Create Complete Graph" 
        
    def Create(self, theGraphEditor):
        theGraphEditor.config(cursor="watch")
        
        dial = Dialog(theGraphEditor, 0, 0, "Create Complete Graph")
        if dial.result is None:
            theGraphEditor.config(cursor="")	    
            return
        
        n=dial.result[0]
        direction=dial.result[2]
        layout=dial.result[3]
        
        G=Graph()
        G.directed=direction
        
        for v in range(0,n):
            G.AddVertex()
            
        Edges=CompleteEdges(G,n,direction)
        
        for e in Edges:
            G.AddEdge(e[0],e[1], initialize_weight=False)
            
        if layout==0:
            if RandomCoords(G):
                DrawNewGraph(theGraphEditor,G,direction)
        else:
            if CircularCoords(G):
                DrawNewGraph(theGraphEditor,G,direction)
                
        theGraphEditor.config(cursor="")
        
        #----------------------------------------------------------------------
class randomGraphCreator(Creator):

    def Name(self):
        return "Create Random Graph" 
        
    def Create(self, theGraphEditor):
        theGraphEditor.config(cursor="watch")
        
        dial = Dialog(theGraphEditor, 0, 1, "Create Random Graph")
        if dial.result is None:
            theGraphEditor.config(cursor="")
            return
            
        n=dial.result[0]
        m=dial.result[1]
        direction=dial.result[2]
        layout=dial.result[3]
        
        G=Graph()
        G.directed=direction
        
        for v in range(0,n):
            G.AddVertex()
            
        Edges=CompleteEdges(G,n,direction)
        
        for i in range(0,m):
            pos=random.randint(0,len(Edges)-1)
            G.AddEdge(Edges[pos][0],Edges[pos][1], initialize_weight=False)
            del Edges[pos]
            
        if layout==0:
            if RandomCoords(G):
                DrawNewGraph(theGraphEditor,G,direction)
        else:
            if CircularCoords(G):
                DrawNewGraph(theGraphEditor,G,direction) 
                
        theGraphEditor.config(cursor="")          
        
        #----------------------------------------------------------------------
class maximalPlanarGraphCreator(Creator):

    def Name(self):
        return "Create Maximal Planar Graph" 
        
    def Create(self, theGraphEditor):
        theGraphEditor.config(cursor="watch")
        
        dial = Dialog(theGraphEditor, 1, 0, "Create Maximal Planar Graph")
        if dial.result is None:
            theGraphEditor.config(cursor="")
            return
            
        n=dial.result[0]
        if n<=1: return
        direction=dial.result[2]
        layout=dial.result[3]
        
        G=Graph()
        G.directed=direction
        
        for v in range(0,n):
            G.AddVertex()
            
        Edges=MaximalPlanarEdges(G,n,direction)
        
        for e in Edges:
            G.AddEdge(e[0],e[1], initialize_weight=False)
            
        if layout==0:
            if RandomCoords(G):
                DrawNewGraph(theGraphEditor,G,direction)
        elif layout==1:
            if CircularCoords(G):
                DrawNewGraph(theGraphEditor,G,direction)
        elif layout==2:
            if FPP_PlanarCoords(G):
                DrawNewGraph(theGraphEditor,G,direction)
        else:
            if Schnyder_PlanarCoords(G):
                DrawNewGraph(theGraphEditor,G,direction)  
                
        theGraphEditor.config(cursor="")
        
        #----------------------------------------------------------------------
from math import log10

class randomPlanarGraphCreator(Creator):

    def Name(self):
        return "Create Random Planar Graph" 
        
    def Create(self, theGraphEditor):
        theGraphEditor.config(cursor="watch")
        
        dial = Dialog(theGraphEditor, 1, 1, "Create Random Planar Graph")
        if dial.result is None:
            theGraphEditor.config(cursor="")
            return
            
        n=dial.result[0]
        if n<=1: return
        m=dial.result[1]
        direction=dial.result[2]
        layout=dial.result[3]
        
        G=Graph()
        G.directed=direction
        
        for v in range(0,n):
            G.AddVertex()
            
        Edges=MaximalPlanarEdges(G,n,direction)
        
        for i in range(0,m):
            pos=random.randint(0,len(Edges)-1)
            G.AddEdge(Edges[pos][0],Edges[pos][1], initialize_weight=False)
            del Edges[pos]
            
        if layout==0:
            if RandomCoords(G):
                DrawNewGraph(theGraphEditor,G,direction)
        elif layout==1:
            if CircularCoords(G):
                DrawNewGraph(theGraphEditor,G,direction)
        elif layout==2:
            if FPP_PlanarCoords(G):
                DrawNewGraph(theGraphEditor,G,direction)
        else:
            if Schnyder_PlanarCoords(G):
                DrawNewGraph(theGraphEditor,G,direction)   
                
        theGraphEditor.config(cursor="")         
        
        #----------------------------------------------------------------------
class TreeDialog(tkSimpleDialog.Dialog):

    def __init__(self, master, visible, Text):
        self.visible=visible
        tkSimpleDialog.Dialog.__init__(self, master, Text)
        
        
    def body(self, master):
        self.resizable(0,0)
        
        self.degree=StringVar()
        self.degree.set("2")
        label = Label(master, text="degree  :", anchor=W)
        label.grid(row=0, column=0, padx=0, pady=2, sticky="w")
        entry=Entry(master, width=6, exportselection=FALSE,
                    textvariable=self.degree)
        entry.selection_range(0,1)
        entry.focus_set()
        entry.grid(row=0,column=1, padx=2, pady=2, sticky="w")
        
        self.height=StringVar()
        self.height.set("0")
        label = Label(master, text="height   :", anchor=W)
        label.grid(row=1, column=0, padx=0, pady=2, sticky="w")
        entry=Entry(master, width=6, exportselection=FALSE,
                    textvariable=self.height)
        entry.selection_range(0,1)
        entry.focus_set()
        entry.grid(row=1,column=1, padx=2, pady=2, sticky="w")
        
        self.number_of_nodes=StringVar()
        self.number_of_nodes.set("1")
        if self.visible:
            label = Label(master, text="#nodes :", anchor=W)
            label.grid(row=2, column=0, padx=0, pady=2, sticky="w")
            entry=Entry(master, width=6, exportselection=FALSE,
                        textvariable=self.number_of_nodes)
            entry.selection_range(0,1)
            entry.focus_set()
            entry.grid(row=2,column=1, padx=2, pady=2, sticky="w")
            
        self.direction=IntVar()
        self.direction.set(0)
        radio=Radiobutton(master, text="Undirected", variable=self.direction,
                          value=0)
        radio.grid(row=0, column=2, padx=2, pady=2, sticky="w") 
        radio=Radiobutton(master, text="Directed", variable=self.direction,
                          value=1)
        radio.grid(row=1, column=2, padx=2, pady=2, sticky="w")
        
        label = Label(master, text=" ")
        label.grid(row=0, column=3, padx=5, pady=2) 
        
        self.layout=IntVar()
        self.layout.set(0)
        radio=Radiobutton(master, text="Randomize Layout",
                          variable=self.layout, value=0,
                          width=17, indicatoron=0, selectcolor="white")
        radio.grid(row=0, column=4, padx=3, pady=2, sticky="w") 
        radio=Radiobutton(master, text="Circular Layout",
                          variable=self.layout, value=1,
                          width=17, indicatoron=0, selectcolor="white")
        radio.grid(row=1, column=4, padx=3, pady=2, sticky="w")
        radio=Radiobutton(master, text="Tree Layout",
                          variable=self.layout, value=2,
                          width=17, indicatoron=0, selectcolor="white")
        radio.grid(row=2, column=4, padx=3, pady=2, sticky="w") 
        radio=Radiobutton(master, text="BFS-Tree Layout",
                          variable=self.layout, value=3,
                          width=17, indicatoron=0, selectcolor="white")
        radio.grid(row=3, column=4, padx=3, pady=2, sticky="w")
        
    def validate(self):
        try:
            d=string.atoi(self.degree.get())
            if d<1 or d>20:
                raise degreeError   
        except:
            showwarning("Please try again !",
                        "min. degree = 1\n"
                        "max. degree = 20")
            return 0
            
        try:
            h=string.atoi(self.height.get())
            if h<0 or h>50:
                raise heightError   
        except:
            showwarning("Please try again !",
                        "min. height = 0\n"
                        "max. height = 50")
            return 0
            
        try:
            n=string.atoi(self.number_of_nodes.get())
        except:
            showwarning("Invalid value !",
                        "Please enter an integer\n"
                        "value for #nodes !")
            return 0
            
        if self.visible:
            if n>1000:
                showwarning("Please try again !",
                            "max. #nodes = 1000")
                return 0
            min_nodes=h+1
            if d==1:
                max_nodes=h+1
            else:
                max_nodes=(float(d)**(h+1)-1)/float(d-1)
            if min_nodes>max_nodes:
                max_nodes=min_nodes
            if max_nodes>1000:
                max_nodes=1000
            if n<min_nodes or n>max_nodes:
                showwarning("Please try again !",
                            "min. #nodes = %i\n"
                            "max. #nodes = %i" %(min_nodes,max_nodes))
                return 0
        else:
            if d==1:
                max_height=100
            else:
                max_height=int(log10(1000)/log10(d))
            if h>max_height:
                showwarning("Please try again !",
                            "max. height = %i" %max_height)
                return 0
                
        self.result=[]
        self.result.append(d)
        self.result.append(h)
        self.result.append(n)
        self.result.append(self.direction.get())
        self.result.append(self.layout.get())
        return self.result
        
        #----------------------------------------------------------------------
class completeTreeCreator(Creator):

    def Name(self):
        return "Create Complete Tree"
        
    def Create(self, theGraphEditor):
        theGraphEditor.config(cursor="watch")
        
        dial = TreeDialog(theGraphEditor, 0, "Create Complete Tree")
        if dial.result is None:
            theGraphEditor.config(cursor="")
            return
            
        degree=dial.result[0]
        height=dial.result[1]
        direction=dial.result[3]
        layout=dial.result[4]
        
        G=Graph()
        G.directed=direction
        
        nodes={}
        nodes[0]=[]
        G.AddVertex()
        nodes[0].append(G.vertices[0])
        for h in range(0,height):
            nodes[h+1]=[]
            for v in nodes[h]:
                for d in range(0,degree):
                    new_v=G.AddVertex()
                    if direction==0: 
                        G.AddEdge(v,new_v, initialize_weight=False)
                    else:
                        if random.randint(0,1):
                            G.AddEdge(v,new_v, initialize_weight=False)
                        else:
                            G.AddEdge(new_v,v, initialize_weight=False)
                    nodes[h+1].append(new_v)
                    
        if layout==0:
            if RandomCoords(G):
                DrawNewGraph(theGraphEditor,G,direction) 
        elif layout==1:
            if CircularCoords(G):
                DrawNewGraph(theGraphEditor,G,direction) 
        elif layout==2:
            if TreeCoords(G,G.vertices[0],"vertical"):
                DrawNewGraph(theGraphEditor,G,direction) 
        else:
            if BFSTreeCoords(G,G.vertices[0],"forward"):
                DrawNewGraph(theGraphEditor,G,direction) 
                
        theGraphEditor.config(cursor="")
        
        #----------------------------------------------------------------------
from math import ceil

class randomTreeCreator(Creator):

    def Name(self):
        return "Create Random Tree"
        
    def Create(self, theGraphEditor):
        theGraphEditor.config(cursor="watch")
        
        dial = TreeDialog(theGraphEditor, 1, "Create Random Tree")
        if dial.result is None:
            theGraphEditor.config(cursor="")
            return
            
        degree=dial.result[0]
        height=dial.result[1]
        n=dial.result[2]
        direction=dial.result[3]
        layout=dial.result[4]
        
        G=Graph()
        G.directed=direction
        
        nodes={}
        nodes[0]=[]
        new_v=G.AddVertex()
        nodes[0].append(new_v)
        children_nr={}
        children_nr[new_v]=0
        for h in range(0,height):
            nodes[h+1]=[]
            if degree==1:
                min_nodes=1
                max_nodes=1
            else:
                min_nodes=max(1,ceil(float(n-G.Order())/
                                     float((float(degree)**(height-h)-1)/
                                           (degree-1))))
                max_nodes=min(n-G.Order()-height+h+1,len(nodes[h])*degree)     
            nodes_nr=random.randint(min_nodes,max_nodes)
            for i in range(0,nodes_nr):
                pos=random.randint(0,len(nodes[h])-1)
                v=nodes[h][pos]
                children_nr[v]=children_nr[v]+1
                if children_nr[v]==degree:
                    del nodes[h][pos]
                new_v=G.AddVertex()
                children_nr[new_v]=0
                if direction==0:
                    G.AddEdge(v,new_v, initialize_weight=False)
                else:
                    if random.randint(0,1):
                        G.AddEdge(v,new_v, initialize_weight=False)
                    else:
                        G.AddEdge(new_v,v, initialize_weight=False)
                nodes[h+1].append(new_v)
                
        if layout==0:
            if RandomCoords(G):
                DrawNewGraph(theGraphEditor,G,direction) 
        elif layout==1:
            if CircularCoords(G):
                DrawNewGraph(theGraphEditor,G,direction) 
        elif layout==2:
            if TreeCoords(G,G.vertices[0],"vertical"):
                DrawNewGraph(theGraphEditor,G,direction) 
        else:
            if BFSTreeCoords(G,G.vertices[0],"forward"):
                DrawNewGraph(theGraphEditor,G,direction) 
                
        theGraphEditor.config(cursor="")
        
        #----------------------------------------------------------------------
from math import ceil

class GridDialog(tkSimpleDialog.Dialog):

    def __init__(self, master, visible, Text):
        self.visible=visible
        tkSimpleDialog.Dialog.__init__(self, master, Text)
        
        
    def body(self, master):
        self.resizable(0,0)
        
        self.xsize=StringVar()
        self.xsize.set("4")
        label = Label(master, text="rows  :", anchor=W)
        label.grid(row=0, column=0, padx=0, pady=2, sticky="w")
        entry=Entry(master, width=6, exportselection=FALSE,
                    textvariable=self.xsize)
        entry.selection_range(0,1)
        entry.focus_set()
        entry.grid(row=0,column=1, padx=2, pady=2, sticky="w")


        self.ysize=StringVar()
        self.ysize.set("6")
        label = Label(master, text="columns:", anchor=W)
        label.grid(row=1, column=0, padx=0, pady=2, sticky="w")
        entry=Entry(master, width=6, exportselection=FALSE,
                    textvariable=self.ysize)
        entry.grid(row=1,column=1, padx=2, pady=2, sticky="w")

        self.deltax=StringVar()
        self.deltax.set("50")
        label = Label(master, text="delta x:", anchor=W)
        label.grid(row=2, column=0, padx=0, pady=2, sticky="w")
        entry=Entry(master, width=6, exportselection=FALSE,
                    textvariable=self.deltax)
        entry.grid(row=2,column=1, padx=2, pady=2, sticky="w")

        self.deltay=StringVar()
        self.deltay.set("50")
        label = Label(master, text="delta y:", anchor=W)
        label.grid(row=3, column=0, padx=0, pady=2, sticky="w")
        entry=Entry(master, width=6, exportselection=FALSE,
                    textvariable=self.deltay)
        entry.grid(row=3,column=1, padx=2, pady=2, sticky="w")

        
    def validate(self):
        try:
            xs=string.atoi(self.xsize.get())
            if xs<1:
                raise degreeError   
        except:
            showwarning("Please try again !",
                        "xsize > 0\n")
            return 0

        try:
            ys=string.atoi(self.ysize.get())
            if ys<1:
                raise degreeError   
        except:
            showwarning("Please try again !",
                        "ysize > 0\n")
            return 0

        try:
            xd=string.atoi(self.deltax.get())
            if xd<1:
                raise degreeError   
        except:
            showwarning("Please try again !",
                        "xdelta > 0\n")
            return 0

        try:
            yd=string.atoi(self.deltay.get())
            if yd<1:
                raise degreeError   
        except:
            showwarning("Please try again !",
                        "ydelta > 0\n")
            return 0

        self.result=[xs,ys,xd,yd]
        return self.result

class rectangularGridGraph(Creator):

    def Name(self):
        return "Create Rectangular Grid Graph"
        
    def Create(self, theGraphEditor):

        theGraphEditor.config(cursor="watch")
        dial = GridDialog(theGraphEditor, 1, "Create Grid Graph")
        if dial.result is None:
            theGraphEditor.config(cursor="")
            return

        maxI = dial.result[0]
        maxJ = dial.result[1]
        deltax = dial.result[2]
        deltay = dial.result[2]
        
        G=Graph()
        G.directed=0
        G.xCoord={}
        G.yCoord={}
        
        nodes = {}
        count = 1
        for i in xrange(maxI):
            for j in xrange(maxJ):
                v = G.AddVertex()
                nodes[(i,j)] = v
                G.xCoord[v] = j * deltax + deltax
                G.yCoord[v] = i * deltay + deltay
                count += 1
                
        for i in xrange(maxI-1):
            for j in xrange(maxJ-1):
                G.AddEdge(nodes[(i,j)], nodes[(i+1,j)], initialize_weight=False)
                G.AddEdge(nodes[(i,j)], nodes[(i,j+1)], initialize_weight=False)
            G.AddEdge(nodes[(i,maxJ-1)], nodes[(i+1,maxJ-1)], initialize_weight=False)
        for  j in xrange(maxJ-1):
            G.AddEdge(nodes[(maxI-1,j)], nodes[(maxI-1,j+1)], initialize_weight=False)
            
        DrawNewGraph(theGraphEditor,G,G.directed) 
        theGraphEditor.config(cursor="")
        
        #----------------------------------------------------------------------
        
""" Here instantiate all the creators you want to make available to
    a client."""
creator = [completeGraphCreator(),
           randomGraphCreator(),
           maximalPlanarGraphCreator(),
           randomPlanarGraphCreator(),
           completeTreeCreator(),
           randomTreeCreator(),
           rectangularGridGraph()]



################################################################################
#
#       This file is part of Gato (Graph Algorithm Toolbox) 
#
#	file:   PlanarEmbedding.py
#	author: Ramazan Buzdemir (buzdemir@zpr.uni-koeln.de)
#
#       Copyright (C) 1998-2015, Alexander Schliep, Winfried Hochstaettler and 
#       Copyright 1998-2001 ZAIK/ZPR, Universitaet zu Koeln
#                                   
#       Contact: alexander@schliep.org, winfried.hochstaettler@fernuni-hagen.de             
#
#       Information: http://gato.sf.net
#
#       This library is free software; you can redistribute it and/or
#       modify it under the terms of the GNU Library General Public
#       License as published by the Free Software Foundation; either
#       version 2 of the License, or (at your option) any later version.
#
#       This library is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#       Library General Public License for more details.
#
#       You should have received a copy of the GNU Library General Public
#       License along with this library; if not, write to the Free
#       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#
#       This file is version $Revision: 670 $ 
#                       from $Date: 2015-01-13 16:04:11 -0500 (Tue, 13 Jan 2015) $
#             last change by $Author: schliep $.
#
################################################################################


###############################################################################
###############################################################################
###############################################################################
#                                                                             #
#                            AN IMPLEMENTATION OF                             #
#                    THE "DE FRAYSSEIX,PACH,POLLACK(FPP)"                     #
#                             AND THE "SCHNYDER"                              #
#                   PLANAR STRAIGHT-LINE EMBEDDING ALGORITHM                  #
#                                                                             #
###############################################################################
#                                                                             #
# References:                                                                 #
#                                                                             #
# [FPP90] H. de Fraysseix, J.Pach, and R.Pollack .                            #
#         "How to draw a planar graph on a grid."                             #
#         Combinatorian, 10:41-51,1990                                        #
# [Sch90] W.Schnyder.                                                         #
#         "Embedding planar graphs on the grid."                              #
#         In 1st Annual ACM-SIAM Symposium on Discrete Algorithms,            #
#         pages 138-14, San Francisco, 1990                                   #
#                                                                             #
###############################################################################



#=============================================================================#
from PlanarityTest import *
from copy import deepcopy
from DataStructures import Stack
from tkMessageBox import showinfo
#=============================================================================#



#=============================================================================#
class pe_Point:

    def __init__(self,xpos,ypos):
        self.x=xpos
        self.y=ypos
        #=============================================================================#
        
        
        
        #=============================================================================#
class pe_Node:

    def __init__(self,x,y):
        self.xpos=x
        self.ypos=y
        self.canOrder=None
        self.t1,self.t2,self.t3=None,None,None 
        self.p1,self.p2,self.p3=None,None,None 
        self.r1,self.r2,self.r3=None,None,None 
        self.xsch,self.ysch=None,None
        self.xfpp,self.yfpp=None,None
        self.adjacentEdges=[]
        self.adjacentNodes=[]
        self.M=[]
        self.oppositeNodes=[]
        self.outface=None
        
        self.path1=[]
        self.path2=[]
        self.path3=[]   
        
    def addEdge(self,e,v):
        self.adjacentEdges.append(e)
        self.adjacentNodes.append(v)
        #=============================================================================#
        
        
        
        #=============================================================================#
class pe_Edge: # directed from p1->p2

    def __init__(self,index_p1,index_p2,ep1,ep2,tf):
        self.p1=index_p1
        self.p2=index_p2
        self.label=None # normal labelling: 1,-1,2,-2,3,-3
        self.original=tf
        self.outface=None
        #=============================================================================#
        
        
        
        #=============================================================================#
class pe_Graph:

    #-------------------------------------------------------------------------
    def printGraph(self):
    
        for i in range(0,len(self.nodes)):
            n=self.nodes[i]
            print "--------Node:",i,"--------------"
            print "xpos=",n.xpos,"ypos=",n.ypos
            print "canOrder=",n.canOrder
            print "t1=",n.t1,"t2=",n.t2,"t3=",n.t3
            print "p1=",n.p1,"p2=",n.p2,"p3=",n.p3
            print "r1=",n.r1,"r2=",n.r2,"r3=",n.r3
            print "xsch=",n.xsch,"ysch=",n.ysch
            print "xfpp=",n.xfpp,"yfpp=",n.yfpp
            print "outface=",n.outface
            
        print
        for i in range(0,len(self.edges)):
            e=self.edges[i]
            print "-------Edge:",i,"---------------"
            print "p1=",e.p1,"p2=",e.p2
            print "label=",e.label
            print "original=",e.original,"outface=",e.outface
            
        print
        for i in range(0,len(self.nodes)):
            n=self.nodes[i]
            
            print "---------------------------"
            print i,":"
            
            print "adjacentEdges:",
            for j in range(0,len(n.adjacentEdges)):
                print n.adjacentEdges[j],
            print
            
            print "adjacentNodes:",
            for j in range(0,len(n.adjacentNodes)):
                print n.adjacentNodes[j],
            print
            
            print "M:",
            for j in range(0,len(n.M)):
                print n.M[j],
            print
            
            print "oppositeNodes:",
            for j in range(0,len(n.oppositeNodes)):
                print n.oppositeNodes[j],
            print
            
            print "path1:",
            for j in range(0,len(n.path1)):
                print n.path1[j],
            print
            
            print "path2:",
            for j in range(0,len(n.path2)):
                print n.path2[j],
            print
            
            print "path3:",
            for j in range(0,len(n.path3)):
                print n.path3[j],
            print
            #-------------------------------------------------------------------------
            
            
            #-------------------------------------------------------------------------
    def __init__(self):
        self.nodes=[]
        self.edges=[]
        
        self.orderK,self.orderIndexVk=None,None
        self.FPPk=None
        self.labelK=None
        self.indexV1,self.indexV2,self.indexV3=-1,-1,-1
        #-------------------------------------------------------------------------
        
        
        #-------------------------------------------------------------------------
    def checkIndex(self,index, p1):
        if index<0:
            tempNode1=pe_Node(p1.x,p1.y)
            self.nodes.append(tempNode1)
            return (len(self.nodes)-1)
        return index
        #-------------------------------------------------------------------------
        
        
        #-------------------------------------------------------------------------
    def storeEdge(self,indexP1,indexP2,p1,p2,tf):
        ep1=pe_Point(self.nodes[indexP1].xpos,self.nodes[indexP1].ypos)
        ep2=pe_Point(self.nodes[indexP2].xpos,self.nodes[indexP2].ypos)
        self.edges.append(pe_Edge(indexP1,indexP2,ep1,ep2,tf))
        #-------------------------------------------------------------------------
        
        
        
        #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # TRIANGULATION
        #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # Algorithm:
        # For each vertex v
        #     for v's each pair of consecutive neighbours u & w
        #             add the edge in
        #             add u into w's incident list in ccw order
        #             add w into u's incident list in ccw order
        #             repeat this procedure
        #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        
        #-------------------------------------------------------------------------
    def isEdge(self,u,w):
        # check if w is in u's adjacentEdges
        if w in u.adjacentNodes: return 1
        return 0
        #-------------------------------------------------------------------------
        
        
        #-------------------------------------------------------------------------
    def adjacentVertex(self,v,e):
        return v.adjacentNodes[e]
        #-------------------------------------------------------------------------
        
        
        #-------------------------------------------------------------------------
    def consider(self):
        for indexV in range(0,len(self.nodes)):
            v=self.nodes[indexV]
            
            if len(v.adjacentEdges)<2: continue
            
            for j in range(0,len(v.adjacentEdges)):
                # get two consective neighbours of v
                indexU=self.adjacentVertex(v,j)
                u=self.nodes[indexU]
                k=j+1
                if k==len(v.adjacentEdges): k=0
                indexW=self.adjacentVertex(v,k)
                w=self.nodes[indexW]
                
                # check if (u, w) is an edge
                if not(self.isEdge(u,indexW)):
                    pointu=pe_Point(u.xpos,u.ypos)
                    pointw=pe_Point(w.xpos,w.ypos)
                    self.storeEdge(indexU,indexW,pointu, pointw,0)
                    
                    tempi1=indexV
                    tempe1=len(self.edges)-1
                    
                    # add u to w's adjacentEdges (with ordering)
                    # add u after v in w's adjacentEdges
                    indexVinW=w.adjacentNodes.index(tempi1)+1
                    w.adjacentEdges.insert(indexVinW,tempe1)
                    w.adjacentNodes.insert(indexVinW,indexU)
                    
                    # add w to u's incitentList (with ordering)
                    # add w before v in u's adjacentEdges
                    indexVinU=u.adjacentNodes.index(tempi1)
                    u.adjacentEdges.insert(indexVinU,tempe1)
                    u.adjacentNodes.insert(indexVinU,indexW)
                    
                    # Don't forget to set original=0
                    self.edges[-1].original=0
                    
                    return 1
        return 0
        #-------------------------------------------------------------------------
        
        
        #-------------------------------------------------------------------------
    def triangulate(self):
        finish=1
        while finish:
            finish=self.consider()
            #-------------------------------------------------------------------------
            
            #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
            
            
            
            #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
            # CANONICAL ORDERING
            #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
            # Algorithm:
            # Pick up a face as outface
            # Assign its vertices with canonical ordering 1,2, and n
            #
            # For k from n-1 to 3 
            #     remove Vk+1 from graph
            #     find all Vk+1's neighbours in the new graph Gk
            #     update the vertices on the outface
            #     assign Vk to one of these neighbours on Ck that
            #                                              is not V1
            #                                              is not V2
            #                                              is not incident to a chord
            #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
            
            #-------------------------------------------------------------------------
    def ordering(self):
        self.orderK=len(self.nodes)
        
        # Now, remove Vn from the graph, and let Vn-1 be the vertex 
        # that is on the outerface and not incident to a chord.
        k=len(self.nodes)
        while k>3:
            self.orderIndexVk=self.order()
            k=k-1
            #-------------------------------------------------------------------------
            
            
            #-------------------------------------------------------------------------
    def initOrder(self):
        # NOTE: initially, all the canOrder are 0
        for i in range(0,len(self.nodes)):
            self.nodes[i].canOrder=0
            
            # Base: find v1, v2, and vn, which define a outerface
        if self.indexV1<0:
            self.indexV1=0
            v1=self.nodes[self.indexV1]
            
            self.indexV2=v1.adjacentNodes[0]
            v2=self.nodes[self.indexV2]
            
            self.indexVn=v1.adjacentNodes[1]
            vk=self.nodes[self.indexVn]
        else:
            v1=self.nodes[self.indexV1]
            v2=self.nodes[self.indexV2]
            vk=self.nodes[self.indexVn]
            
        v1.canOrder=1
        v2.canOrder=2
        vk.canOrder=len(self.nodes)
        
        # initialize all the outface to 0
        for j in range(0,len(self.nodes)): 
            self.nodes[j].outface=0
            
        self.orderK=len(self.nodes)
        self.orderIndexVk=self.indexVn
        return self.indexVn
        #-------------------------------------------------------------------------
        
        
        #-------------------------------------------------------------------------
        # Now, remove Vn from the graph, and let Vn-1 be the vertex 
        # that is on the outerface and not incident to a chord
    def order(self):
        if self.orderK>3:
            v1=self.nodes[self.indexV1]
            v2=self.nodes[self.indexV2]
            vk=self.nodes[self.orderIndexVk]
            # "remove" Vk from the graph
            # find the neighbours of Vk, that have canOrder number < k
            # define they are on the outface
            for i in range(0,len(vk.adjacentNodes)):
                neighbour=self.nodes[vk.adjacentNodes[i]]
                
                if neighbour.canOrder<self.orderK:
                    neighbour.outface=1
                    
                    # find the node that is not v1, v2, and not incident to any chord,
                    # let it be Vk-1
            found=0
            j=0
            while not(found) and j<len(self.nodes):
                candidate=self.nodes[j]
                if (candidate.outface and candidate!=v1 and
                    candidate!=v2 and candidate.canOrder<self.orderK):
                    # if it only has 2 neighbours on the outface,
                    # we set it to be Vk-1
                    count=0
                    for i in range(0,len(candidate.adjacentNodes)):
                        checkIndex=candidate.adjacentNodes[i]
                        checkNode=self.nodes[checkIndex]
                        if (checkNode.outface and
                            (checkNode.canOrder<self.orderK)):
                            count=count+1
                    if count==2:
                        found=1
                        vk=candidate
                        candidate.canOrder=self.orderK-1
                        
                        self.orderK=self.orderK-1
                        self.orderIndexVk=j
                        return j
                j=j+1
        return -1
        #-------------------------------------------------------------------------
        
        #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        
        
        
        #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # EDGE LABELLING 
        #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # Algorithm:
        # In triangle V1, V2, V3 (canonical order)
        #     label V3->V1 with 1
        #     label V3->V2 with 2
        #
        # For k from 3 to n-1   
        #     add Vk+1 to graph Gk
        #     find all Vk+1's neighbours in Gk in order
        #     label the left most edge from top to bottom with 1
        #     label the right most edge from top to bottom with 2
        #     label the rest from bottom to top with 3
        #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        
        #-------------------------------------------------------------------------
    def labelling(self):
        self.initLabel()
        #steps
        for k in range(3,len(self.nodes)):
            self.labelK=k
            self.labelStep()
            # looking for v4, v5, ..., vn
            #-------------------------------------------------------------------------
            
            
            #-------------------------------------------------------------------------
            # label  label if the edge is indexP1 -> indexP2
            #       -label if the edge is indexP2 -> indexP1
    def labelEdge(self,indexP1,indexP2,label):
        e=self.edges[0]
        
        if e.p1==indexP1 and e.p2==indexP2:
            self.edges[0].label=label
            return
        if e.p1==indexP2 and e.p2==indexP1:
            self.edges[0].label=-label 
            return
            
        for i in range(1,len(self.edges)):
            e=self.edges[i]
            if e.p1==indexP1 and e.p2==indexP2:
                e.label=label
                return
            if e.p1==indexP2 and e.p2==indexP1:
                e.label=-label
                return
                #-------------------------------------------------------------------------
                
                
                #-------------------------------------------------------------------------
    def findIndexOfVk(self,k):
        indexVk=-1
        i=0
        
        while indexVk<0:
            if self.nodes[i].canOrder==k:
                return i
            i=i+1
        return -1
        #-------------------------------------------------------------------------
        
        
        #-------------------------------------------------------------------------
    def initLabel(self):
        for j in range(0,len(self.edges)):
            self.edges[j].label=0
            
        self.indexV3=self.findIndexOfVk(3)
        
        # find v1, v2, v3
        v1=self.nodes[self.indexV1]
        v2=self.nodes[self.indexV2]
        v3=self.nodes[self.indexV3]
        
        # labelling should be done at the same time as FPP is running
        # (because we need the outface information)
        # but we are doing this separately, for the sak of clearness
        
        # label V3 -> V1 by 1
        self.labelEdge(self.indexV3,self.indexV1,1)
        
        # label V3 -> V2 by 2
        self.labelEdge(self.indexV3,self.indexV2,2)
        
        self.labelK = 3
        #-------------------------------------------------------------------------
        
        
        #-------------------------------------------------------------------------
    def labelStep(self):
        k=self.labelK
        n=len(self.nodes)
        
        if k<n:
            indexVkplus1=self.findIndexOfVk(k+1)
            vkplus1=self.nodes[indexVkplus1]
            
            # labelling should be done at the same time as FPP is running
            # (because we need the outface information)
            # case 1: vk+1 is "to the right" of vk
            # case 2: vk+1 is "to the left" of vk
            # case 3: vk+1 "covers" vk
            # all make the first element in Vk+1's oppositeNodes label 1, 
            #   	   last                                        2,
            #  		   rest        	   			       3.
            # oppositeNodes is done in FPP
            
            first=vkplus1.oppositeNodes[0]
            self.labelEdge(indexVkplus1,first,1)
            
            last=vkplus1.oppositeNodes[-1]
            self.labelEdge(indexVkplus1,last,2)
            
            if len(vkplus1.oppositeNodes)>2:
                for l in range(1,len(vkplus1.oppositeNodes)-1):
                    self.labelEdge(indexVkplus1,vkplus1.oppositeNodes[l],-3)
                    
            self.labelK=self.labelK+1
        return self.labelK
        #-------------------------------------------------------------------------
        
        #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        
        
        
        #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # FPP
        #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # Algorithm:
        # 3. Initialize x,y coordinates and M for V1,V2, and V3
        #                               v1.M={v1,v2,v3}
        #                               v2.M={v2}
        #                               v3.M={v2,v3}
        # 4. In the canonical order, for each vertex
        #       1. find the vertices on the outface in order
        #       2. shift vertices in the subset M of Wp+1 and Wq
        #       3. calculate the x,y coordinates of Vk+1
        #       4. updating M for all the outface vertices
        #                               wi.M=wi.M+{vk+1}  for i<=p
        #                               vk+1.M=wp+1.M+{vk+1}
        #                               wj.M=wj.M  for j>=q
        #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        
        #-------------------------------------------------------------------------
    def FPP(self):
        self.initFPP()
        
        # steps
        for k in range(3,len(self.nodes)):
            self.FPPk=k
            self.FPPstep()
            #-------------------------------------------------------------------------
            
            
            #-------------------------------------------------------------------------
    def initFPP(self):
        self.indexV3=self.findIndexOfVk(3)
        
        # initialize all the outface to 0
        for j in range(0,len(self.nodes)):
            self.nodes[j].outface=0
            self.nodes[j].xfpp=0
            self.nodes[j].yfpp=0
            self.nodes[j].M=[]
            self.nodes[j].oppositeNodes=[]
            
            # find v1, v2, v3
        v1=self.nodes[self.indexV1]
        v2=self.nodes[self.indexV2]
        v3=self.nodes[self.indexV3] 
        
        # basic
        v1.xfpp=0; v1.yfpp=0
        v2.xfpp=2; v2.yfpp=0
        v3.xfpp=1; v3.yfpp=1
        
        # v1.M={v1,v2,v3} v2.M={v2} v3.M={v2,v3}
        v1.M.append(self.indexV1)
        v1.M.append(self.indexV2)
        v1.M.append(self.indexV3)
        
        v2.M.append(self.indexV2)
        
        v3.M.append(self.indexV2)
        v3.M.append(self.indexV3) 
        
        self.nodes[self.indexV1].outface=1
        self.nodes[self.indexV2].outface=1
        self.nodes[self.indexV3].outface=1
        
        self.FPPk=3
        #-------------------------------------------------------------------------
        
        
        #-------------------------------------------------------------------------
    def FPPstep(self):
        k=self.FPPk
        n=len(self.nodes)
        
        if k<n:
            indexVkplus1=self.findIndexOfVk(k+1)
            vkplus1=self.nodes[indexVkplus1]
            
            # find the vertices on the outerface of Gk,
            # and in order of p, p+1, ..., q
            # find the neighbours of v(k+1) with CanOrder <= k,
            # and sort them according to their xfpp
            for i in range(0,len(vkplus1.adjacentNodes)):
                neighbour=self.nodes[vkplus1.adjacentNodes[i]]
                if neighbour.canOrder<=k:
                    insertPlace=-1
                    j=0
                    while insertPlace<0 and j<len(vkplus1.oppositeNodes):
                        if (neighbour.xfpp <
                            self.nodes[vkplus1.oppositeNodes[j]].xfpp):
                            insertPlace=j
                        j=j+1
                        
                    if insertPlace==-1:
                        vkplus1.oppositeNodes.append(
                            self.nodes.index(neighbour))
                    else:
                        vkplus1.oppositeNodes.insert(insertPlace,
                                                 self.nodes.index(neighbour))
                        
                        # find the vertices on the outface
            self.nodes[indexVkplus1].outface=1
            if len(vkplus1.oppositeNodes)>2:
                for i in range(1,len(vkplus1.oppositeNodes)-1):
                    temp=vkplus1.oppositeNodes[i]
                    self.nodes[temp].outface=0
                    
                    # shift all vertices in w(p+1).M right by 1 unit
            indexWpplus1=vkplus1.oppositeNodes[1]
            w=self.nodes[indexWpplus1]
            for i in range(0,len(w.M)):
                temp=w.M[i]
                self.nodes[temp].xfpp=self.nodes[temp].xfpp+1
                
                # shift all vertices in w(q).M right by 1 unit
            Wq=self.nodes[vkplus1.oppositeNodes[-1]]
            for i in range(0,len(Wq.M)): 
                self.nodes[Wq.M[i]].xfpp=self.nodes[Wq.M[i]].xfpp+1
                
                # add in v(k+1)
                
            Wp=self.nodes[vkplus1.oppositeNodes[0]]
            x1=Wp.xfpp
            y1=Wp.yfpp
            x2=Wq.xfpp
            y2=Wq.yfpp
            
            vkplus1.xfpp=(x1+x2+y2-y1)/2
            vkplus1.yfpp=(x2-x1+y2+y1)/2
            
            # update M
            # wi.M = wi.M + v(k+1)  for i<=p
            for i in range(0,n):
                wi=self.nodes[i]
                if wi.outface  and  wi.xfpp<w.xfpp and i!=indexVkplus1:
                    wi.M.append(indexVkplus1)
                    
                    # v(k+1).M = w(p+1).M + v(k+1)
            vkplus1.M.append(indexVkplus1)
            for i in range(0,len(w.M)):
                vkplus1.M.append(w.M[i])
                
                
            self.FPPk=self.FPPk+1
            
        return self.FPPk
        #-------------------------------------------------------------------------
        
        #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        
        
        
        #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # SCHNYDER
        #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        # Algorithm:
        # 4. Calculate for each interior vertex v
        #              pathi : vertices on the i-path from v to v1, v2, or vn
        #              pn : number of vertices on the i-path starting at v
        #              ti : number of vertices in the subtree of Ti rooted at v
        #              ri : number of vertices in region Ri(v) for v
        # 5. Calculate barycentric representation for each v: vi'=ri - pi-1
        #                                            v->(v1',v2',v3')/(n-1)
        #    A barycentric representation of a graph G is 
        #    an injective function v->(v1,v2,v3) that satisfies:
        #      a.) v1+v2+v3=1 for all v
        #      b.) for each edge (x,y) and each vertex z not x or y,
        #          there is some k (k=1,2 or 3) such that xk < zk and yk < zk   
        #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        
        #-------------------------------------------------------------------------
    def calculateP(self,path123):
        for i in range(0,len(self.nodes)):
            v=self.nodes[i] 
            finalNode=0
            
            if (v.canOrder!=1 and v.canOrder!=2 and 
                v.canOrder!=len(self.nodes)):
                # v is an interior vertex
                if path123==1:
                    v.p1=1
                    v.path1.append(i)
                    finalNode=1
                if path123==2:
                    v.p2=1
                    v.path2.append(i)
                    finalNode=2
                if path123==3:
                    v.p3=1
                    v.path3.append(i)
                    finalNode=len(self.nodes)
                    
                vNext=v
                while vNext.canOrder!=finalNode:
                    j=0
                    found=0
                    while (not(found) and j<len(vNext.adjacentEdges)):
                        curEdge=self.edges[vNext.adjacentEdges[j]]
                        curLabel=curEdge.label
                        # HIER IST IRGENDWO EIN FEHLER !!!!!!
                        if ( ((curLabel==path123) and 
                              (curEdge.p1==self.nodes.index(vNext))) or 
                             ((curLabel==-path123) and
                              (curEdge.p2==self.nodes.index(vNext))) ):
                            found=1
                            vNextIndex=vNext.adjacentNodes[j]
                            vNext=self.nodes[vNextIndex]
                            
                            if path123==1:
                                v.p1=v.p1+1
                                v.path1.append(vNextIndex)
                            if path123==2:
                                v.p2=v.p2+1
                                v.path2.append(vNextIndex)
                            if path123==3:
                                v.p3=v.p3+1
                                v.path3.append(vNextIndex)
                        j=j+1
                        #-------------------------------------------------------------------------
                        
                        
                        #-------------------------------------------------------------------------
    def traverse(self,label,v,count):
        for j in range(0,len(v.adjacentEdges)):
            curEdge=self.edges[v.adjacentEdges[j]]
            curLabel=curEdge.label
            if ( ((curLabel==-label) and 
                  (curEdge.p1==self.nodes.index(v))) or
                 ((curLabel==label) and
                  (curEdge.p2==self.nodes.index(v))) ):
                vNextIndex=v.adjacentNodes[j]
                vNext=self.nodes[vNextIndex]
                count=count+1
                count=self.traverse(label,vNext,count)
        return count
        #-------------------------------------------------------------------------
        
        
        #-------------------------------------------------------------------------
    def Schnyder(self):
        # we need to compute p1, p2, p3 and t1, t2, t3 and
        # r1, r2, r3 for each vertex
    
        # Initialize the data
        for i in range(0,len(self.nodes)):
            tempnn1=self.nodes[i]
            tempnn1.p1=0
            tempnn1.p2=0
            tempnn1.p3=0
            tempnn1.t1=0
            tempnn1.t2=0
            tempnn1.t3=0
            tempnn1.r1=0
            tempnn1.r2=0
            tempnn1.r3=0
            tempnn1.xsch=0
            tempnn1.ysch=0
            tempnn1.path1=[]
            tempnn1.path2=[]
            tempnn1.path3=[]
            
            # find those for v1, v2, and vn
        v1=self.nodes[self.indexV1]
        v2=self.nodes[self.indexV2]
        vn=self.nodes[self.indexVn]
        v1.t1=0; v1.t2=1; v1.t3=1;
        v2.t1=1; v2.t2=0; v2.t3=1;
        vn.t1=1; vn.t2=1; vn.t3=0;
        v1.p1=0; vn.p1=1;
        v2.p2=0; vn.p2=1;
        
        v1.xsch=len(self.nodes)-1; v1.ysch=0;
        v2.xsch=0; v2.ysch=len(self.nodes)-1;
        vn.xsch=0; vn.ysch = 0;
        
        
        # can we get p1/p2/p3 while doing ordering or labelling???
        # Not really!!! We cannot get p3 easily
        
        # calculate p1, p2, p3 by going through path1, path2, path3
        self.calculateP(1)
        self.calculateP(2)
        self.calculateP(3)
        
        # calculate t1, t2, t3
        # exterior vertices are done in ordering
        # for each interior vertex v
        for i in range(0,len(self.nodes)):
            v=self.nodes[i]
            
            if (v.canOrder!=1 and v.canOrder!=2 and
                v.canOrder!=len(self.nodes)):
                # v is an interior vertex
                v.t1=self.traverse(1,v,1) # Itself is in the subtree
                v.t2=self.traverse(2,v,1) # Itself is in the subtree
                v.t3=self.traverse(3,v,1) # Itself is in the subtree
                
                # calculate r1, r2, r3
                # we need 3 vectors in each vertex to store the path 1,2,3
                # v.ri = ti of all vertices on P(i+1)+ti of all vertices on P(i-1)-ti
        for i in range(0,len(self.nodes)):
            v=self.nodes[i]
            
            if (v.canOrder!=1 and v.canOrder!=2 and
                v.canOrder!=len(self.nodes)):
                # v is an interior vertex
                v.r1=0; v.r2=0; v.r3=0;
                
                # r1
                for j in range(0,len(v.path2)):
                    onPath=self.nodes[v.path2[j]]
                    v.r1=v.r1+onPath.t1
                for j in range(0,len(v.path3)):
                    onPath=self.nodes[v.path3[j]]
                    v.r1=v.r1+onPath.t1
                v.r1=v.r1-v.t1
                
                # r2
                for j in range(0,len(v.path1)):
                    onPath=self.nodes[v.path1[j]]
                    v.r2=v.r2+onPath.t2
                for j in range(0,len(v.path3)):
                    onPath=self.nodes[v.path3[j]]
                    v.r2=v.r2+onPath.t2
                v.r2=v.r2-v.t2
                
                # r3
                for j in range(0,len(v.path1)):
                    onPath=self.nodes[v.path1[j]]
                    v.r3=v.r3+onPath.t3
                for j in range(0,len(v.path2)):
                    onPath=self.nodes[v.path2[j]]
                    v.r3=v.r3+onPath.t3
                v.r3=v.r3-v.t3
                
                
                # The coordinates of each vertex is (v'1, v'2), 
                # where v'i = ri - p(i-1)
                # exterior vertices are done in ordering
                # for each interior vertex v
        for i in range(0,len(self.nodes)):
            v=self.nodes[i]
            
            if (v.canOrder!=1 and v.canOrder!=2 and
                v.canOrder!=len(self.nodes)):
                # v is an interior vertex
                v.xsch=(v.r1-v.p3)
                v.ysch=(v.r2-v.p1)
                #-------------------------------------------------------------------------
                
                #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                
                
                #=============================================================================#
                # LOAD GRAPH
                
def load_graph(InGraph):

    if InGraph.Order()<3:
        return 0
        
    ccwOrderedEdges=planarity_test(InGraph)
    if not(ccwOrderedEdges):
        showinfo("Planarity Test", "Graph is NOT PLANAR!")
        return 0
        
    graph1=pe_Graph()
    
    i=0
    NodeIndex={}
    for v in InGraph.vertices:
        NodeIndex[v]=i
        i=i+1
        
    for i in range(0,InGraph.Order()):
        graph1.nodes.append(pe_Node(i,i))
        
    EdgeIndex={}
    for i in range(0,len(ccwOrderedEdges)):
        n1=NodeIndex[ccwOrderedEdges[i][0]]
        n2=NodeIndex[ccwOrderedEdges[i][1]]
        ccwOrderedEdges[i]=(n1,n2)
        EdgeIndex[(n1,n2)]=None
        
    i=0
    for e in ccwOrderedEdges:
        if EdgeIndex[e]==None:
            EdgeIndex[e]=i
            EdgeIndex[(e[1],e[0])]=i
            i=i+1
            p1=pe_Point(e[0],e[0])
            p2=pe_Point(e[1],e[1])
            tempe1=pe_Edge(e[0],e[1],p1,p2,1)
            graph1.edges.append(tempe1)
            
        graph1.nodes[e[0]].addEdge(EdgeIndex[e],e[1])
        
    return graph1
    #=============================================================================#
    
    
    
    #=============================================================================#
def FPP_PlanarCoords(G, rect=None): # (2n-4)*(n-2) GRID
# Algorithm: 
# 1. Triangulate orginal graph
# 2. Canonical order all vertices
# 3. Initialize x,y coordinates and M for V1,V2, and V3
#                               v1.M={v1,v2,v3}
#                               v2.M={v2}
#                               v3.M={v2,v3}
# 4. In the canonical order, for each vertex
#       1. find the vertices on the outface in order
#       2. shift vertices in the subset M of Wp+1 and Wq
#       3. calculate the x,y coordinates of Vk+1
#       4. updating M for all the outface vertices
#                               wi.M=wi.M+{vk+1}  for i<=p
#                               vk+1.M=wp+1.M+{vk+1}
#                               wj.M=wj.M  for j>=q

    #-------------------------------------------------------------------------
    # LOAD GRAPH
    graph=load_graph(G)
    if graph==0: return 0
    #-------------------------------------------------------------------------
    
    
    #-------------------------------------------------------------------------
    # 1.TRIANGULATION
    graph.triangulate()
    #-------------------------------------------------------------------------
    
    
    #-------------------------------------------------------------------------
    # 2.CANONICAL ORDERING
    graph.initOrder()
    graph.ordering()
    #-------------------------------------------------------------------------
    
    
    #-------------------------------------------------------------------------
    # 3+4. FPP 
    graph.FPP()
    #-------------------------------------------------------------------------
    
    
    #-------------------------------------------------------------------------
    if rect is None:
        from Embedder import Rect
        rect = Rect()
    # COORDINATES
    G.xCoord={}
    G.yCoord={}
    n=len(graph.nodes)
    for i in range(0,n):
        G.xCoord[G.vertices[i]]=graph.nodes[i].xfpp*float(
            (rect.height()-100)/(2*n-4))+50
        G.yCoord[G.vertices[i]]=rect.width()-(graph.nodes[i].yfpp*float(
            (rect.width()-100)/(n-2))+50)
    return 1
    #-------------------------------------------------------------------------
    
    #=============================================================================#
    
    
    
    #=============================================================================#
def Schnyder_PlanarCoords(G, rect=None): # (n-1)*(n-1) GRID
# Algorithm: 
# 1. Triangulate orginal graph
# 2. Canonical order all vertices
# 3. Normal label interior edges of G to i->Ti (i=1,2,3)
# 4. Calculate for each interior vertex v
#              pathi : vertices on the i-path from v to v1, v2, or vn
#              pn : number of vertices on the i-path starting at v
#              ti : number of vertices in the subtree of Ti rooted at v
#              ri : number of vertices in region Ri(v) for v
# 5. Calculate barycentric representation for each v: vi'=ri - pi-1
#                                            v->(v1',v2',v3')/(n-1)
#    A barycentric representation of a graph G is 
#    an injective function v->(v1,v2,v3) that satisfies:
#      a.) v1+v2+v3=1 for all v
#      b.) for each edge (x,y) and each vertex z not x or y,
#          there is some k (k=1,2 or 3) such that xk < zk and yk < zk       

    #-------------------------------------------------------------------------
    # LOAD GRAPH
    graph=load_graph(G)
    if graph==0: return 0
    #-------------------------------------------------------------------------
    
    
    #-------------------------------------------------------------------------
    # 1.TRIANGULATION
    graph.triangulate()
    #-------------------------------------------------------------------------
    
    
    #-------------------------------------------------------------------------
    # 2.CANONICAL ORDERING
    graph.initOrder()
    graph.ordering()
    #-------------------------------------------------------------------------
    
    
    #-------------------------------------------------------------------------
    # 3. EDGE LABELLING 
    graph.FPP() # outfaces
    graph.labelling()
    #-------------------------------------------------------------------------
    
    
    #-------------------------------------------------------------------------
    # 4+5. Schnyder
    graph.Schnyder()
    #-------------------------------------------------------------------------
    
    
    #-------------------------------------------------------------------------
    if rect is None:
        from Embedder import Rect
        rect = Rect()
    # COORDINATES
    G.xCoord={}
    G.yCoord={}
    n=len(graph.nodes)
    for i in range(0,n):
        G.xCoord[G.vertices[i]]=graph.nodes[i].xsch*float(
            (rect.height()-100)/(n-1))+50
        G.yCoord[G.vertices[i]]=rect.width()-(graph.nodes[i].ysch*float(
            (rect.width()-100)/(n-1))+50)
    return 1
    #-------------------------------------------------------------------------
    
    #=============================================================================#



################################################################################
#
#       This file is part of Gato (Graph Animation Toolbox) 
#
#	file:   Graph.py
#	author: Alexander Schliep (alexander@schliep.org)
#
#       Copyright (C) 1998-2015, Alexander Schliep, Winfried Hochstaettler and
#       Copyright 1998-2001 ZAIK/ZPR, Universitaet zu Koeln
#
#       Contact: alexander@schliep.org, winfried.hochstaettler@fernuni-hagen.de
#
#       Information: http://gato.sf.net
#
#       This library is free software; you can redistribute it and/or
#       modify it under the terms of the GNU Library General Public
#       License as published by the Free Software Foundation; either
#       version 2 of the License, or (at your option) any later version.
#
#       This library is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#       Library General Public License for more details.
#
#       You should have received a copy of the GNU Library General Public
#       License along with this library; if not, write to the Free
#       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#
#       This file is version $Revision: 670 $
#                       from $Date: 2015-01-13 16:04:11 -0500 (Tue, 13 Jan 2015) $
#             last change by $Author: schliep $
#
################################################################################

import types
import StringIO
from string import split
import string
from GatoGlobals import *
import Graph
from DataStructures import Point2D, VertexLabeling, EdgeLabeling, EdgeWeight, VertexWeight, Queue
import logging
log = logging.getLogger("Gato")


################################################################################
#
# Syntactic Sugar
#
################################################################################
def Vertices(G):
    """ Returns the vertices of G. Hide method call """
    return G.vertices
    
def Edges(G):
    """ Returns the edges of G. Hide method call """
    return G.Edges()
    
    
    ################################################################################
    #
    # Basic algorithms
    #
    ################################################################################
    
def BFS(G,root,direction='forward'):
    """ Calculate BFS distances and predecessor without showing animations. 
        If G is directed, direction does matter:
    
        - 'forward'  BFS will use outgoing edges
        - 'backward' BFS will use incoming edges
    
        It uses gInfinity (from GatoGlobals.py) as infinite distance.
        returns (dist,pred) """
    
    Q = Queue()
    d = {}
    pred = {}
    
    for v in G.vertices:
        d[v] = gInfinity
    d[root] = 0
    pred[root] = root
    
    Q.Append(root)
    
    while Q.IsNotEmpty():
        v = Q.Top()
        if G.QDirected() == 1 and direction == 'backward':
            nbh = G.InNeighbors(v)
        else:
            nbh = G.Neighborhood(v)
            
        for w in nbh:
            if d[w] == gInfinity:
                d[w] = d[v] + 1
                pred[w] = v
                Q.Append(w)
                
    return (d,pred)
    
    
def ConnectedComponents(G):
    """ Compute the connected components of the undirected graph G.
        Returns a list of lists of vertices. """
    
    
    result = []
    visited = {}
    for v in G.vertices:
        visited[v] = None
        
    for root in G.vertices:
        if visited[root] is not None:
            continue
        else: # Found a new component
            component = [root]
            visited[root] = 1
            
            Q = Queue()
            Q.Append(root)
            
            while Q.IsNotEmpty():
                v = Q.Top()
                nbh = G.Neighborhood(v)
                for w in nbh:
                    if visited[w] == None:
                        visited[w] = 1
                        Q.Append(w)
                        component.append(w)
                        
            result.append(component)
            
    return result
    

def FindBipartitePartitions(G):
    """ Finds the two partitions of a bipartite graph G and returns
        two vertex sets. If G is not bipartite, it returns two empty
        sets.
    """
    Q = Queue()
    V = set()
    W = set()
    dist = {}

    for v in G.vertices:
        if v not in dist:
            dist[v] = 0
            Q.Append(v)
            V.add(v)

            while Q.IsNotEmpty():
                v = Q.Top()

                for w in G.Neighborhood(v):
                    if w not in dist:
                        dist[w] = dist[v] + 1
                        Q.Append(w)

                        if dist[w] % 2 == 1:
                            W.add(w)
                        else:
                            V.add(w)
                    else:
                        if (dist[w] + dist[v]) % 2 == 0:
                            # Found an incompatible edge
                            return (set(), set())                            
    return (V, W)


    
    ################################################################################
    #
    # GraphInformer
    #
    ################################################################################
    
    
class GraphInformer:
    """ Provides information about edges and vertices of a graph.
        Used as argument for GraphDisplay.RegisterGraphInformer() """
    
    def __init__(self,G):
        self.G = G
        self.info = ""
        
    def DefaultInfo(self):
        """ Provide an default text which is shown when no edge/vertex
            info is displayed """  
        return self.info
        
    def VertexInfo(self,v):
        """ Provide an info text for vertex v """
        t = self.G.GetEmbedding(v)
        return "Vertex %d at position (%d,%d)" % (v, int(t.x), int(t.y))
        
    def EdgeInfo(self,tail,head):
        """ Provide an info text for edge (tail,head)  """        
        return "Edge (%d,%d)" % (tail, head) 
        
    def SetDefaultInfo(self, info=""):
        self.info = info
        
        
class WeightedGraphInformer(GraphInformer):
    """ Provides information about weighted edges and vertices of a graph.
        Used as argument for GraphDisplay.RegisterGraphInformer() """
    
    def __init__(self,G,weightDesc="weight"):
        """ G is the graph we want to supply information about and weightDesc
            a textual interpretation of the weight """
        GraphInformer.__init__(self,G)
        self.weightDesc = weightDesc
        
    def EdgeInfo(self,tail,head):
        """ Provide an info text for weighted edge (tail,head)  """  
        # How to handle undirected graph
        if self.G.QDirected() == 0:
            try:
                w = self.G.GetEdgeWeight(0,tail, head)
            except KeyError:
                w = self.G.GetEdgeWeight(0, head, tail)
        else:
            w = self.G.GetEdgeWeight(0, tail, head)
        if self.G.edgeWeights[0].QInteger():
            return "Edge (%d,%d) %s: %d" % (tail, head, self.weightDesc, w) 
        else:
            return "Edge (%d,%d) %s: %f" % (tail, head, self.weightDesc, w) 
            
            
class MSTGraphInformer(WeightedGraphInformer):
    def __init__(self,G,T):
        WeightedGraphInformer.__init__(self,G)
        self.T = T
        
    def DefaultInfo(self):
        """ Provide an default text which is shown when no edge/vertex
            info is displayed """  
        return "Tree has %d vertices and weight %5.2f" % (self.T.Order(),self.T.Weight())
        
        
class FlowGraphInformer(GraphInformer):
    def __init__(self,G,flow):
        GraphInformer.__init__(self,G)
        self.flow   = flow
        self.cap    = flow.cap
        self.res    = flow.res
        self.excess = flow.excess
        
    def EdgeInfo(self,v,w):
        return "Edge (%d,%d) - flow: %d of %d" % (v,w, self.flow[(v,w)], self.cap[(v,w)])
        
    def VertexInfo(self,v):
        tmp = self.excess[v]
        if tmp == gInfinity:
            str1 = "Infinity"
        elif tmp == -gInfinity:
            str1 = "-Infinity"
        else:
            str1 = "%d"%tmp

        return "Vertex %d - excess: %s" % (v, str1)

class ResidualGraphInformer(FlowGraphInformer):

    def EdgeInfo(self,v,w):
        return "Edge (%d,%d) - residual capacity: %d" % (v, w, self.res[(v,w)])
        
        ################################################################################
        #
        # FILE I/O
        #
        ################################################################################
        
def OpenCATBoxGraph(_file):
    """ Reads in a graph from file fileName. File-format is supposed
        to be from old CATBOX++ (*.cat) """
    G = Graph.Graph()
    E = VertexLabeling()
    W = EdgeWeight(G)
    L = VertexLabeling()
    
    # get file from name or file object
    graphFile=None
    if type(_file) in types.StringTypes:
        graphFile = open(_file, 'r')
    elif type(_file)==types.FileType or issubclass(_file.__class__,StringIO.StringIO):
        graphFile=_file
    else:
        raise Exception("got wrong argument")
        
    lineNr = 1
    
    firstVertexLineNr = -1    
    lastVertexLineNr  = -1
    firstEdgeLineNr   = -1
    lastEdgeLineNr    = -1
    intWeights        = 0
    
    while 1:
    
        line = graphFile.readline()
        
        if not line:
            break
            
        if lineNr == 2: # Read directed and euclidian
            splitLine = split(line[:-1],';')	    
            G.directed = eval(split(splitLine[0],':')[1])
            G.simple = eval(split(splitLine[1],':')[1])
            G.euclidian = eval(split(splitLine[2],':')[1])
            intWeights = eval(split(splitLine[3],':')[1])
            nrOfEdgeWeights = eval(split(splitLine[4],':')[1])
            nrOfVertexWeights = eval(split(splitLine[5],':')[1])
            for i in xrange(nrOfEdgeWeights):
                G.edgeWeights[i] = EdgeWeight(G)
            for i in xrange(nrOfVertexWeights):
                G.vertexWeights[i] = VertexWeight(G)
                
                
        if lineNr == 5: # Read nr of vertices
            nrOfVertices = eval(split(line[:-2],':')[1]) # Strip of "\n" and ; 
            firstVertexLineNr = lineNr + 1
            lastVertexLineNr  = lineNr + nrOfVertices
            
        if  firstVertexLineNr <= lineNr and lineNr <= lastVertexLineNr: 
            splitLine = split(line[:-1],';')
            v = G.AddVertex()
            x = eval(split(splitLine[1],':')[1])
            y = eval(split(splitLine[2],':')[1])
            for i in xrange(nrOfVertexWeights):
                w = eval(split(splitLine[3+i],':')[1])
                G.vertexWeights[i][v] = w
                
            E[v] = Point2D(x,y)
            
        if lineNr == lastVertexLineNr + 1: # Read Nr of edges
            nrOfEdges = eval(split(line[:-2],':')[1]) # Strip of "\n" and ; 
            firstEdgeLineNr = lineNr + 1
            lastEdgeLineNr  = lineNr + nrOfEdges
            
        if firstEdgeLineNr <= lineNr and lineNr <= lastEdgeLineNr: 
            splitLine = split(line[:-1],';')
            h = eval(split(splitLine[0],':')[1])
            t = eval(split(splitLine[1],':')[1])
            G.AddEdge(t,h,False)
            for i in xrange(nrOfEdgeWeights):
                G.edgeWeights[i][(t,h)] = eval(split(splitLine[3+i],':')[1])
                
        lineNr = lineNr + 1
        
    graphFile.close()
    
    for v in G.vertices:
        L[v] = v
        
    G.embedding = E
    G.labeling  = L
    if intWeights:
        G.Integerize('all')
        for i in xrange(nrOfVertexWeights):
            G.vertexWeights[i].Integerize()
            
    return G
    
def SaveCATBoxGraph(G, _file):
    """ Save graph to file fileName in file-format from old CATBOX++ (*.cat) """
    
    # get file from name or file object
    file=None
    if type(_file) in types.StringTypes:
        file = open(_file, 'w')
    elif type(_file)==types.FileType or issubclass(_file.__class__,StringIO.StringIO):
        file=_file
    else:
        raise Exception("got wrong argument")
        
    nrOfVertexWeights = len(G.vertexWeights.keys())
    nrOfEdgeWeights = len(G.edgeWeights.keys())
    integerEdgeWeights = G.edgeWeights[0].QInteger()
    
    file.write("graph:\n")
    file.write("dir:%d; simp:%d; eucl:%d; int:%d; ew:%d; vw:%d;\n" %
               (G.QDirected(), G.simple, G.QEuclidian(), integerEdgeWeights,
               nrOfEdgeWeights, nrOfVertexWeights))
    file.write("scroller:\n")
    file.write("vdim:1000; hdim:1000; vlinc:10; hlinc:10; vpinc:50; hpinc:50;\n")
    file.write("vertices:" + `G.Order()` + ";\n")

    # Being paranoid here: We force edge weights to be euclidean 
    if G.QEuclidian():
        G.Euclidify()
    
    # Force continous numbering of vertices
    count = 1
    save = {}
    for v in G.vertices:
        save[v] = count
        count = count + 1
        file.write("n:%d; x:%d; y:%d;" % (save[v], G.embedding[v].x, G.embedding[v].y))
        for i in xrange(nrOfVertexWeights):
            if integerEdgeWeights: # XXX
                file.write(" w:%d;" % int(round(G.vertexWeights[i][v])))
            else:
                file.write(" w:%d;" % G.vertexWeights[i][v])	    
        file.write("\n")
        
    file.write("edges:" + `G.Size()` + ";\n")
    for tail in G.vertices:
        for head in G.OutNeighbors(tail):
            file.write("h:%d; t:%d; e:2;" % (save[head], save[tail]))
            
            for i in xrange(nrOfEdgeWeights):
                if integerEdgeWeights:
                    file.write(" w:%d;" % int(round(G.edgeWeights[i][(tail,head)])))
                else:
                    file.write(" w:%f;" % G.edgeWeights[i][(tail,head)])
                    
            file.write("\n")
            
            #### GML
            
def ParseGML(file):

    retval = []
    
    while 1:
    
        line = file.readline() 
        
        if not line:
            return retval
            
        token = filter(lambda x: x != '', split(line[:-1],"[\t ]*"))
        
        if len(token) == 1 and token[0] == ']':
            return retval
            
        elif len(token) == 2:
        
            if token[1] == '[':
                retval.append((token[0], ParseGML(file)))
            else:
                retval.append((token[0], token[1]))
                
        else:
            log.error("Serious format error line %s:" % line)
            
            
def PairListToDictionary(l):
    d = {}
    for i in xrange(len(l)):
        d[l[i][0]] = l[i][1]
    return d
    
    
    
def OpenGMLGraph(fileName):
    """ Reads in a graph from file fileName. File-format is supposed
        to be GML (*.gml) """
    G = Graph.Graph()
    G.directed = 0
    E = VertexLabeling()
    W = EdgeWeight(G)
    L = VertexLabeling()
    VLabel = VertexLabeling()
    ELabel = EdgeLabeling()
    
    file = open(fileName, 'r')
    g = ParseGML(file)
    file.close()
    
    if g[0][0] != 'graph':
        log.error("Serious format error in %s. first key is not graph" % fileName)
        return
    else:
        l = g[0][1]
        for i in xrange(len(l)):
        
            key   = l[i][0]
            value = l[i][1]
            
            if key == 'node':
            
                d = PairListToDictionary(value)
                v = G.AddVertex()
                
                try:
                    VLabel[v] = eval(d['label'])
                    P = PairListToDictionary(d['graphics'])
                    E[v] = Point2D(eval(P['x']), eval(P['y']))
                    
                except:
                    d = None 
                    P = None
                    
            elif key == 'edge':
            
                d = PairListToDictionary(value)
                
                try:
                    s = eval(d['source'])
                    t = eval(d['target'])
                    G.AddEdge(s,t)
                    ELabel[(s,t)] = eval(d['label'])
                    W[(s,t)] = 0
                except:
                    d = None 
                    
            elif key == 'directed':
                G.directed = 1 
                
    for v in G.vertices:
        L[v] = v
        
    G.embedding = E
    G.labeling  = L
    G.nrEdgeWeights = 1
    G.edgeWeights[0] = W
    G.vertexAnnotation = VLabel
    G.edgeAnnotation = ELabel
    
    return G
    
    
    
def OpenDotGraph(fileName):
    """ Reads in a graph from file fileName. File-format is supposed
        to be dot (*.dot) used in """
    G = Graph.Graph()
    G.directed = 1
    E = VertexLabeling()
    W = EdgeWeight(G)
    L = VertexLabeling()
    VLabel = VertexLabeling()
    ELabel = EdgeLabeling()
    
    import re
    file = open(fileName, 'r')
    lines = file.readlines()
    file.close()
    
    dot2graph = {}
    
    for l in lines[3:]:
        items = string.split(l)
        if len(items) < 2:
            break
        if items[1] != '->':
            v = G.AddVertex()
            dot_v = int(items[0])
            L[v] = "%d" % dot_v
            dot2graph[dot_v] = v
            m = re.search('label=("[^"]+")', l)
            VLabel[v] = m.group(1)[1:-1]
            m = re.search('pos="(\d+),(\d+)"', l)
            x = int(m.group(1))
            y = int(m.group(2))
            E[v] = Point2D(x,y)
        else:
            m = re.search('(\d+) -> (\d+)', l)
            v = dot2graph[int(m.group(1))]
            w = dot2graph[int(m.group(2))]
            m = re.search('label=("[^"]+")', l)
            #print l
            #print v,w,m.group(1)
            G.AddEdge(v,w)
            weight = float(m.group(1)[1:-1])
            W[(v,w)] = weight
            ELabel[(v,w)] = "%0.2f" % weight
            
    G.embedding = E
    G.labeling  = L
    G.nrEdgeWeights = 1
    G.edgeWeights[0] = W
    G.vertexAnnotation = VLabel
    G.edgeAnnotation = ELabel
    return G


def WeightMatrix(G=None, fromFileName=None, noEdgeWeight=999999.9):
    import numpy
    if fromFileName:
        G = OpenCATBoxGraph(fromFileName)
    directed = G.QDirected()
    A = numpy.ones((G.Order(),G.Order())) * noEdgeWeight
    for i in range(G.Order()):
        A[i,i] = 0
    for i,j in G.Edges():
        A[i-1,j-1] = G.GetEdgeWeight(0,i,j)
        if not directed:
            A[i-1,j-1] = G.GetEdgeWeight(0,i,j)
    return A



################################################################################
#
#       This file is part of Gato (Graph Animation Toolbox) 
#
#	file:   GatoUtil.py
#	author: Alexander Schliep (alexander@schliep.org)
#
#       Copyright (C) 1998-2015, Alexander Schliep, Winfried Hochstaettler and 
#       Copyright 1998-2001 ZAIK/ZPR, Universitaet zu Koeln
#                                   
#       Contact: alexander@schliep.org, winfried.hochstaettler@fernuni-hagen.de             
#
#       Information: http://gato.sf.net
#
#       This library is free software; you can redistribute it and/or
#       modify it under the terms of the GNU Library General Public
#       License as published by the Free Software Foundation; either
#       version 2 of the License, or (at your option) any later version.
#
#       This library is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#       Library General Public License for more details.
#
#       You should have received a copy of the GNU Library General Public
#       License along with this library; if not, write to the Free
#       Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#
#       This file is version $Revision: 670 $ 
#                       from $Date: 2015-01-13 16:04:11 -0500 (Tue, 13 Jan 2015) $
#             last change by $Author: schliep $.
#
################################################################################

from Tkinter import *
    
def extension(pathAndFile):
    """ Return ext if path/filename.ext is given """
    import string
    return string.split(stripPath(pathAndFile),".")[-1]
    
def stripPath(pathAndFile):
    """ Return filename.ext if path/filename.ext is given """
    import os 
    return os.path.split(pathAndFile)[1]
    
def orthogonal(u):
    """ Return a unit length vector (v1,v2) which has an angle of
        90 degrees clockwise to the vector u = (u1,u2) """
    from math import sqrt
    (u1,u2) = u
    length = sqrt(u1**2 + u2**2)
    if length < 0.001:
        length = 0.001
    u1 = u1 / length
    u2 = u2 / length
    return (-u2,u1)
      
def ArgMin(list,val):
    """ Returns the element e of list for which val[e] is minimal.
    """
    values = [val[e] for e in list]
    return list[values.index(min(values))]
    
def ArgMax(list,val):
    """ Returns the element e of list for which val[e] is maximal """
    values = [val[e] for e in list]
    return list[values.index(max(values))]

            
class ImageCache:
    """ Provides a global cache for PhotoImages displayed in the 
        application. Singleton Pattern

        ic = ImageCache()

        # image_data is base64 encoded gif file
        ic.AddImage('some/path/image.gif', image_data)

        ...image_create('insert', image=ic['some/path/image.gif']
    """
    images = None	
    
    def __init__(self):
        if ImageCache.images == None:
            ImageCache.images = {}
            
    def __getitem__(self, relURL):
        """ Given a relative URL to an image file return the 
            corresponding PhotoImage. """
        try:    
            if relURL not in self.images.keys():
                ImageCache.images[relURL] = PhotoImage(file=relURL)
            return ImageCache.images[relURL]
        except IndexError, IOError:
            import logging
            log = logging.getLogger("GatoUtil.py")
            log.exception("Error finding image %s" % relURL)
            
    def AddImage(self, relURL, imageData):
        ImageCache.images[relURL] = PhotoImage(data=imageData)
